@@ -1,6 +1,7 @@
 import Vue from 'vue';
+import canaryCalloutMixin from 'ee_else_ce/environments/mixins/canary_callout_mixin';
 import environmentsComponent from './components/environments_app.vue';
-import { convertPermissionToBoolean } from '../lib/utils/common_utils';
+import { parseBoolean } from '../lib/utils/common_utils';
 import Translate from '../vue_shared/translate';
 
 Vue.use(Translate);
@@ -11,6 +12,7 @@ export default () =>
     components: {
       environmentsComponent,
     },
+    mixins: [canaryCalloutMixin],
     data() {
       const environmentsData = document.querySelector(this.$options.el).dataset;
 
@@ -19,9 +21,8 @@ export default () =>
         newEnvironmentPath: environmentsData.newEnvironmentPath,
         helpPagePath: environmentsData.helpPagePath,
         cssContainerClass: environmentsData.cssClass,
-        canCreateEnvironment: convertPermissionToBoolean(environmentsData.canCreateEnvironment),
-        canCreateDeployment: convertPermissionToBoolean(environmentsData.canCreateDeployment),
-        canReadEnvironment: convertPermissionToBoolean(environmentsData.canReadEnvironment),
+        canCreateEnvironment: parseBoolean(environmentsData.canCreateEnvironment),
+        canReadEnvironment: parseBoolean(environmentsData.canReadEnvironment),
       };
     },
     render(createElement) {
@@ -32,8 +33,8 @@ export default () =>
           helpPagePath: this.helpPagePath,
           cssContainerClass: this.cssContainerClass,
           canCreateEnvironment: this.canCreateEnvironment,
-          canCreateDeployment: this.canCreateDeployment,
           canReadEnvironment: this.canReadEnvironment,
+          ...this.canaryCalloutProps,
         },
       });
     },
