@@ -2,16 +2,16 @@
 
 module Applications
   class CreateService
-    # rubocop: disable CodeReuse/ActiveRecord
+    attr_reader :current_user, :params
+
     def initialize(current_user, params)
       @current_user = current_user
-      @params = params.except(:ip_address)
+      @params = params.except(:ip_address) # rubocop: disable CodeReuse/ActiveRecord
     end
-    # rubocop: enable CodeReuse/ActiveRecord
 
     # EE would override and use `request` arg
     def execute(request)
-      Doorkeeper::Application.create(@params)
+      Doorkeeper::Application.create(params)
     end
   end
 end
