@@ -36,6 +36,10 @@ module Clusters
       case clusterable
       when ::Project
         { cluster_type: :project_type, projects: [clusterable] }
+      when ::Group
+        { cluster_type: :group_type, groups: [clusterable] }
+      else
+        raise NotImplementedError
       end
     end
 
