@@ -6,7 +6,7 @@ module QA
           element :push_events, "event_filter_link EventFilter::PUSH, _('Push events')" # rubocop:disable QA/ElementWithPattern
         end
 
-        def go_to_push_events
+        def click_push_events
           click_on 'Push events'
         end
       end
