@@ -1,13 +1,11 @@
+# frozen_string_literal: true
+
 module QA::Page
   module Project::Job
     class Show < QA::Page::Base
       COMPLETED_STATUSES = %w[passed failed canceled blocked skipped manual].freeze # excludes created, pending, running
       PASSED_STATUS = 'passed'.freeze
 
-      view 'app/assets/javascripts/jobs/components/job_app.vue' do
-        element :loading_animation
-      end
-
       view 'app/assets/javascripts/jobs/components/job_log.vue' do
         element :build_trace
       end
@@ -16,18 +14,17 @@ module QA::Page
         element :status_badge
       end
 
-      def completed?
-        COMPLETED_STATUSES.include?(status_badge)
+      view 'app/assets/javascripts/jobs/components/stages_dropdown.vue' do
+        element :pipeline_path
       end
 
-      def passed?
+      def successful?(timeout: 60)
+        raise "Timed out waiting for the build trace to load" unless loaded?
+        raise "Timed out waiting for the status to be a valid completed state" unless completed?(timeout: timeout)
+
         status_badge == PASSED_STATUS
       end
 
-      def trace_loading?
-        has_element?(:loading_animation)
-      end
-
       # Reminder: You may wish to wait for a particular job status before checking output
       def output
         find_element(:build_trace).text
@@ -35,6 +32,18 @@ module QA::Page
 
       private
 
+      def loaded?(wait: 60)
+        wait(reload: true, max: wait, interval: 1) do
+          has_element?(:build_trace, wait: 1)
+        end
+      end
+
+      def completed?(timeout: 60)
+        wait(reload: false, max: timeout) do
+          COMPLETED_STATUSES.include?(status_badge)
+        end
+      end
+
       def status_badge
         find_element(:status_badge).text
       end
