@@ -9,15 +9,7 @@ module QA
             element :edit_page_link, 'Edit' # rubocop:disable QA/ElementWithPattern
           end
 
-          def go_to_new_page
-            click_on 'New page'
-          end
-
-          def got_to_view_history_page
-            click_on 'Page history'
-          end
-
-          def go_to_edit_page
+          def click_edit
             click_on 'Edit'
           end
         end
