@@ -5,13 +5,13 @@ module QA
     module Project
       module Wiki
         class Show < Page::Base
-          include Page::Component::ClonePanel
+          include Page::Component::LegacyClonePanel
 
           view 'app/views/projects/wikis/pages.html.haml' do
             element :clone_repository_link, 'Clone repository' # rubocop:disable QA/ElementWithPattern
           end
 
-          def go_to_clone_repository
+          def click_clone_repository
             click_on 'Clone repository'
           end
         end
