@@ -3,7 +3,7 @@
 module QA
   context 'Create' do
     describe 'Git push over HTTP', :ldap_no_tls do
-      it 'user using a personal access token pushes code to the repository'  do
+      it 'user using a personal access token pushes code to the repository' do
         Runtime::Browser.visit(:gitlab, Page::Main::Login)
         Page::Main::Login.perform(&:sign_in_using_credentials)
 
@@ -22,7 +22,8 @@ module QA
         end
 
         push.project.visit!
-        Page::Project::Show.perform(&:wait_for_push)
+
+        Page::Project::Show.perform(&:wait_for_viewers_to_load)
 
         expect(page).to have_content('README.md')
         expect(page).to have_content('This is a test project')
