@@ -3,17 +3,16 @@
 module QA
   context 'Create' do
     describe 'Git push over HTTP', :ldap_no_tls do
-      it 'user pushes code to the repository'  do
+      it 'user pushes code to the repository' do
         Runtime::Browser.visit(:gitlab, Page::Main::Login)
-        Page::Main::Login.act { sign_in_using_credentials }
+        Page::Main::Login.perform(&:sign_in_using_credentials)
 
-        Resource::Repository::ProjectPush.fabricate! do |push|
+        project_push = Resource::Repository::ProjectPush.fabricate! do |push|
           push.file_name = 'README.md'
           push.file_content = '# This is a test project'
           push.commit_message = 'Add README.md'
         end
-
-        Page::Project::Show.act { wait_for_push }
+        project_push.project.visit!
 
         expect(page).to have_content('README.md')
         expect(page).to have_content('This is a test project')
