@@ -3,21 +3,15 @@
 module QA
   context 'Create' do
     describe 'Wiki management' do
-      def login
-        Runtime::Browser.visit(:gitlab, Page::Main::Login)
-        Page::Main::Login.act { sign_in_using_credentials }
-      end
-
       def validate_content(content)
         expect(page).to have_content('Wiki was successfully updated')
         expect(page).to have_content(/#{content}/)
       end
 
-      before do
-        login
-      end
-
       it 'user creates, edits, clones, and pushes to the wiki' do
+        Runtime::Browser.visit(:gitlab, Page::Main::Login)
+        Page::Main::Login.perform(&:sign_in_using_credentials)
+
         wiki = Resource::Wiki.fabricate! do |resource|
           resource.title = 'Home'
           resource.content = '# My First Wiki Content'
@@ -26,7 +20,7 @@ module QA
 
         validate_content('My First Wiki Content')
 
-        Page::Project::Wiki::Edit.act { go_to_edit_page }
+        Page::Project::Wiki::Edit.perform(&:click_edit)
         Page::Project::Wiki::New.perform do |page|
           page.set_content("My Second Wiki Content")
           page.save_changes
@@ -40,7 +34,7 @@ module QA
           push.file_content = '# My Third Wiki Content'
           push.commit_message = 'Update Home.md'
         end
-        Page::Project::Menu.act { click_wiki }
+        Page::Project::Menu.perform(&:click_wiki)
 
         expect(page).to have_content('My Third Wiki Content')
       end
