@@ -1,11 +1,12 @@
 # frozen_string_literal: true
 
 module QA
-  context 'Release' do
+  # Failure issue: https://gitlab.com/gitlab-org/quality/staging/issues/26
+  context 'Release', :quarantine do
     describe 'Deploy key creation' do
       it 'user adds a deploy key' do
         Runtime::Browser.visit(:gitlab, Page::Main::Login)
-        Page::Main::Login.act { sign_in_using_credentials }
+        Page::Main::Login.perform(&:sign_in_using_credentials)
 
         key = Runtime::Key::RSA.new
         deploy_key_title = 'deploy key title'
@@ -16,7 +17,13 @@ module QA
           resource.key = deploy_key_value
         end
 
-        expect(deploy_key.fingerprint).to eq(key.fingerprint)
+        expect(deploy_key.fingerprint).to eq key.fingerprint
+
+        Page::Project::Settings::Repository.perform do |setting|
+          setting.expand_deploy_keys do |keys|
+            expect(keys).to have_key(deploy_key_title, key.fingerprint)
+          end
+        end
       end
     end
   end
