@@ -114,7 +114,7 @@ describe ApplicationController do
       skip_before_action :authenticate_user!, only: :index
 
       def index
-        render text: 'authenticated'
+        render html: 'authenticated'
       end
     end
 
@@ -401,7 +401,7 @@ describe ApplicationController do
   context 'terms' do
     controller(described_class) do
       def index
-        render text: 'authenticated'
+        render html: 'authenticated'
       end
     end
 
@@ -423,7 +423,7 @@ describe ApplicationController do
         enforce_terms
       end
 
-      it 'redirects if the user did not accept the terms'  do
+      it 'redirects if the user did not accept the terms' do
         get :index
 
         expect(response).to have_gitlab_http_status(302)
@@ -444,7 +444,7 @@ describe ApplicationController do
       attr_reader :last_payload
 
       def index
-        render text: 'authenticated'
+        render html: 'authenticated'
       end
 
       def append_info_to_payload(payload)
@@ -460,6 +460,14 @@ describe ApplicationController do
       expect(controller.last_payload.has_key?(:response)).to be_falsey
     end
 
+    it 'does log correlation id' do
+      Gitlab::CorrelationId.use_id('new-id') do
+        get :index
+      end
+
+      expect(controller.last_payload).to include('correlation_id' => 'new-id')
+    end
+
     context '422 errors' do
       it 'logs a response with a string' do
         response = spy(ActionDispatch::Response, status: 422, body: 'Hello world', content_type: 'application/json', cookies: {})
@@ -499,7 +507,7 @@ describe ApplicationController do
   describe '#access_denied' do
     controller(described_class) do
       def index
-        access_denied!(params[:message])
+        access_denied!(params[:message], params[:status])
       end
     end
 
@@ -511,12 +519,20 @@ describe ApplicationController do
       get :index
 
       expect(response).to have_gitlab_http_status(404)
+      expect(response).to render_template('errors/not_found')
     end
 
     it 'renders a 403 when a message is passed to access denied' do
-      get :index, message: 'None shall pass'
+      get :index, params: { message: 'None shall pass' }
 
       expect(response).to have_gitlab_http_status(403)
+      expect(response).to render_template('errors/access_denied')
+    end
+
+    it 'renders a status passed to access denied' do
+      get :index, params: { status: 401 }
+
+      expect(response).to have_gitlab_http_status(401)
     end
   end
 
@@ -534,34 +550,18 @@ describe ApplicationController do
     end
 
     context 'html' do
-      subject { get :index, text: "hi \255" }
+      subject { get :index, params: { text: "hi \255" } }
 
       it 'renders 412' do
-        if Gitlab.rails5?
-          expect { subject }.to raise_error(ActionController::BadRequest)
-        else
-          subject
-
-          expect(response).to have_gitlab_http_status(412)
-          expect(response).to render_template :precondition_failed
-        end
+        expect { subject }.to raise_error(ActionController::BadRequest)
       end
     end
 
     context 'js' do
-      subject { get :index, text: "hi \255", format: :js }
+      subject { get :index, format: :js, params: { text: "hi \255" } }
 
       it 'renders 412' do
-        if Gitlab.rails5?
-          expect { subject }.to raise_error(ActionController::BadRequest)
-        else
-          subject
-
-          json_response = JSON.parse(response.body)
-
-          expect(response).to have_gitlab_http_status(412)
-          expect(json_response['error']).to eq('Invalid UTF-8')
-        end
+        expect { subject }.to raise_error(ActionController::BadRequest)
       end
     end
   end
@@ -665,6 +665,14 @@ describe ApplicationController do
 
         expect(response.headers['Cache-Control']).to eq 'max-age=0, private, must-revalidate, no-store'
       end
+
+      it 'does not set the "no-store" header for XHR requests' do
+        sign_in(user)
+
+        get :index, xhr: true
+
+        expect(response.headers['Cache-Control']).to eq 'max-age=0, private, must-revalidate'
+      end
     end
   end
 end
