@@ -1,6 +1,8 @@
 require 'spec_helper'
 
 describe GroupsController do
+  include ExternalAuthorizationServiceHelpers
+
   let(:user) { create(:user) }
   let(:admin) { create(:admin) }
   let(:group) { create(:group, :public) }
@@ -15,7 +17,7 @@ describe GroupsController do
     it 'renders the new page' do
       sign_in(member)
 
-      get :new, parent_id: group.id
+      get :new, params: { parent_id: group.id }
 
       expect(response).to render_template(:new)
     end
@@ -25,35 +27,60 @@ describe GroupsController do
     it 'renders the 404 page' do
       sign_in(member)
 
-      get :new, parent_id: group.id
+      get :new, params: { parent_id: group.id }
 
       expect(response).not_to render_template(:new)
       expect(response.status).to eq(404)
     end
   end
 
+  shared_examples 'details view' do
+    it { is_expected.to render_template('groups/show') }
+
+    context 'as atom' do
+      let!(:event) { create(:event, project: project) }
+      let(:format) { :atom }
+
+      it { is_expected.to render_template('groups/show') }
+
+      it 'assigns events for all the projects in the group' do
+        subject
+        expect(assigns(:events)).to contain_exactly(event)
+      end
+    end
+  end
+
   describe 'GET #show' do
     before do
       sign_in(user)
       project
     end
 
-    context 'as atom' do
-      it 'assigns events for all the projects in the group' do
-        create(:event, project: project)
+    let(:format) { :html }
 
-        get :show, id: group.to_param, format: :atom
+    subject { get :show, params: { id: group.to_param }, format: format }
 
-        expect(assigns(:events)).not_to be_empty
-      end
+    it_behaves_like 'details view'
+  end
+
+  describe 'GET #details' do
+    before do
+      sign_in(user)
+      project
     end
+
+    let(:format) { :html }
+
+    subject { get :details, params: { id: group.to_param }, format: format }
+
+    it_behaves_like 'details view'
   end
 
   describe 'GET edit' do
     it 'sets the badge API endpoint' do
       sign_in(owner)
 
-      get :edit, id: group.to_param
+      get :edit, params: { id: group.to_param }
 
       expect(assigns(:badge_api_endpoint)).not_to be_nil
     end
@@ -102,7 +129,7 @@ describe GroupsController do
           create(:event, project: project)
         end
 
-        get :activity, id: group.to_param, format: :json
+        get :activity, params: { id: group.to_param }, format: :json
 
         expect(response).to have_gitlab_http_status(200)
         expect(json_response['count']).to eq(3)
@@ -120,7 +147,7 @@ describe GroupsController do
               owner.update_attribute(:can_create_group, can_create_group_status)
               sign_in(owner)
 
-              post :create, group: { parent_id: group.id, path: 'subgroup' }
+              post :create, params: { group: { parent_id: group.id, path: 'subgroup' } }
 
               expect(response).to be_redirect
               expect(response.body).to match(%r{http://test.host/#{group.path}/subgroup})
@@ -134,7 +161,7 @@ describe GroupsController do
 
               previous_group_count = Group.count
 
-              post :create, group: { parent_id: group.id, path: 'subgroup' }
+              post :create, params: { group: { parent_id: group.id, path: 'subgroup' } }
 
               expect(response).to render_template(:new)
               expect(Group.count).to eq(previous_group_count)
@@ -157,7 +184,7 @@ describe GroupsController do
         it 'creates the Group' do
           original_group_count = Group.count
 
-          post :create, group: { path: 'subgroup' }
+          post :create, params: { group: { path: 'subgroup' } }
 
           expect(Group.count).to eq(original_group_count + 1)
           expect(response).to be_redirect
@@ -172,7 +199,7 @@ describe GroupsController do
         it 'does not create the Group' do
           original_group_count = Group.count
 
-          post :create, group: { path: 'subgroup' }
+          post :create, params: { group: { path: 'subgroup' } }
 
           expect(Group.count).to eq(original_group_count)
           expect(response).to render_template(:new)
@@ -215,36 +242,35 @@ describe GroupsController do
 
     context 'sorting by votes' do
       it 'sorts most popular issues' do
-        get :issues, id: group.to_param, sort: 'upvotes_desc'
+        get :issues, params: { id: group.to_param, sort: 'upvotes_desc' }
         expect(assigns(:issues)).to eq [issue_2, issue_1]
       end
 
       it 'sorts least popular issues' do
-        get :issues, id: group.to_param, sort: 'downvotes_desc'
+        get :issues, params: { id: group.to_param, sort: 'downvotes_desc' }
         expect(assigns(:issues)).to eq [issue_2, issue_1]
       end
     end
 
     context 'searching' do
-      # Remove as part of https://gitlab.com/gitlab-org/gitlab-ce/issues/52271
       before do
-        stub_feature_flags(use_cte_for_group_issues_search: false)
+        stub_feature_flags(attempt_group_search_optimizations: true)
       end
 
       it 'works with popularity sort' do
-        get :issues, id: group.to_param, search: 'foo', sort: 'popularity'
+        get :issues, params: { id: group.to_param, search: 'foo', sort: 'popularity' }
 
         expect(assigns(:issues)).to eq([issue_1])
       end
 
       it 'works with priority sort' do
-        get :issues, id: group.to_param, search: 'foo', sort: 'priority'
+        get :issues, params: { id: group.to_param, search: 'foo', sort: 'priority' }
 
         expect(assigns(:issues)).to eq([issue_1])
       end
 
       it 'works with label priority sort' do
-        get :issues, id: group.to_param, search: 'foo', sort: 'label_priority'
+        get :issues, params: { id: group.to_param, search: 'foo', sort: 'label_priority' }
 
         expect(assigns(:issues)).to eq([issue_1])
       end
@@ -265,12 +291,12 @@ describe GroupsController do
 
     context 'sorting by votes' do
       it 'sorts most popular merge requests' do
-        get :merge_requests, id: group.to_param, sort: 'upvotes_desc'
+        get :merge_requests, params: { id: group.to_param, sort: 'upvotes_desc' }
         expect(assigns(:merge_requests)).to eq [merge_request_2, merge_request_1]
       end
 
       it 'sorts least popular merge requests' do
-        get :merge_requests, id: group.to_param, sort: 'downvotes_desc'
+        get :merge_requests, params: { id: group.to_param, sort: 'downvotes_desc' }
         expect(assigns(:merge_requests)).to eq [merge_request_2, merge_request_1]
       end
     end
@@ -281,7 +307,7 @@ describe GroupsController do
       it 'returns 404' do
         sign_in(create(:user))
 
-        delete :destroy, id: group.to_param
+        delete :destroy, params: { id: group.to_param }
 
         expect(response.status).to eq(404)
       end
@@ -294,12 +320,12 @@ describe GroupsController do
 
       it 'schedules a group destroy' do
         Sidekiq::Testing.fake! do
-          expect { delete :destroy, id: group.to_param }.to change(GroupDestroyWorker.jobs, :size).by(1)
+          expect { delete :destroy, params: { id: group.to_param } }.to change(GroupDestroyWorker.jobs, :size).by(1)
         end
       end
 
       it 'redirects to the root path' do
-        delete :destroy, id: group.to_param
+        delete :destroy, params: { id: group.to_param }
 
         expect(response).to redirect_to(root_path)
       end
@@ -312,7 +338,7 @@ describe GroupsController do
     end
 
     it 'updates the path successfully' do
-      post :update, id: group.to_param, group: { path: 'new_path' }
+      post :update, params: { id: group.to_param, group: { path: 'new_path' } }
 
       expect(response).to have_gitlab_http_status(302)
       expect(controller).to set_flash[:notice]
@@ -320,11 +346,18 @@ describe GroupsController do
 
     it 'does not update the path on error' do
       allow_any_instance_of(Group).to receive(:move_dir).and_raise(Gitlab::UpdatePathError)
-      post :update, id: group.to_param, group: { path: 'new_path' }
+      post :update, params: { id: group.to_param, group: { path: 'new_path' } }
 
       expect(assigns(:group).errors).not_to be_empty
       expect(assigns(:group).path).not_to eq('new_path')
     end
+
+    it 'updates the project_creation_level successfully' do
+      post :update, params: { id: group.to_param, group: { project_creation_level: ::Gitlab::Access::MAINTAINER_PROJECT_ACCESS } }
+
+      expect(response).to have_gitlab_http_status(302)
+      expect(group.reload.project_creation_level).to eq(::Gitlab::Access::MAINTAINER_PROJECT_ACCESS)
+    end
   end
 
   describe '#ensure_canonical_path' do
@@ -336,7 +369,7 @@ describe GroupsController do
       context 'when requesting groups at the root path' do
         before do
           allow(request).to receive(:original_fullpath).and_return("/#{group_full_path}")
-          get :show, id: group_full_path
+          get :show, params: { id: group_full_path }
         end
 
         context 'when requesting the canonical path with different casing' do
@@ -383,7 +416,7 @@ describe GroupsController do
           context 'non-show path' do
             context 'with exactly matching casing' do
               it 'does not redirect' do
-                get :issues, id: group.to_param
+                get :issues, params: { id: group.to_param }
 
                 expect(response).not_to have_gitlab_http_status(301)
               end
@@ -391,7 +424,7 @@ describe GroupsController do
 
             context 'with different casing' do
               it 'redirects to the correct casing' do
-                get :issues, id: group.to_param.upcase
+                get :issues, params: { id: group.to_param.upcase }
 
                 expect(response).to redirect_to(issues_group_path(group.to_param))
                 expect(controller).not_to set_flash[:notice]
@@ -402,7 +435,7 @@ describe GroupsController do
           context 'show path' do
             context 'with exactly matching casing' do
               it 'does not redirect' do
-                get :show, id: group.to_param
+                get :show, params: { id: group.to_param }
 
                 expect(response).not_to have_gitlab_http_status(301)
               end
@@ -410,7 +443,7 @@ describe GroupsController do
 
             context 'with different casing' do
               it 'redirects to the correct casing at the root path' do
-                get :show, id: group.to_param.upcase
+                get :show, params: { id: group.to_param.upcase }
 
                 expect(response).to redirect_to(group)
                 expect(controller).not_to set_flash[:notice]
@@ -423,7 +456,7 @@ describe GroupsController do
           let(:redirect_route) { group.redirect_routes.create(path: 'old-path') }
 
           it 'redirects to the canonical path' do
-            get :issues, id: redirect_route.path
+            get :issues, params: { id: redirect_route.path }
 
             expect(response).to redirect_to(issues_group_path(group.to_param))
             expect(controller).to set_flash[:notice].to(group_moved_message(redirect_route, group))
@@ -433,7 +466,7 @@ describe GroupsController do
             let(:redirect_route) { group.redirect_routes.create(path: 'http') }
 
             it 'does not modify the requested host' do
-              get :issues, id: redirect_route.path
+              get :issues, params: { id: redirect_route.path }
 
               expect(response).to redirect_to(issues_group_path(group.to_param))
               expect(controller).to set_flash[:notice].to(group_moved_message(redirect_route, group))
@@ -445,7 +478,7 @@ describe GroupsController do
             let(:redirect_route) { group.redirect_routes.create(path: 'oups') }
 
             it 'does not modify the /groups part of the path' do
-              get :issues, id: redirect_route.path
+              get :issues, params: { id: redirect_route.path }
 
               expect(response).to redirect_to(issues_group_path(group.to_param))
               expect(controller).to set_flash[:notice].to(group_moved_message(redirect_route, group))
@@ -457,7 +490,7 @@ describe GroupsController do
             let(:redirect_route) { group.redirect_routes.create(path: 'oups/oup') }
 
             it 'does not modify the /groups part of the path' do
-              get :issues, id: redirect_route.path
+              get :issues, params: { id: redirect_route.path }
 
               expect(response).to redirect_to(issues_group_path(group.to_param))
               expect(controller).to set_flash[:notice].to(group_moved_message(redirect_route, group))
@@ -469,13 +502,13 @@ describe GroupsController do
       context 'for a POST request' do
         context 'when requesting the canonical path with different casing' do
           it 'does not 404' do
-            post :update, id: group.to_param.upcase, group: { path: 'new_path' }
+            post :update, params: { id: group.to_param.upcase, group: { path: 'new_path' } }
 
             expect(response).not_to have_gitlab_http_status(404)
           end
 
           it 'does not redirect to the correct casing' do
-            post :update, id: group.to_param.upcase, group: { path: 'new_path' }
+            post :update, params: { id: group.to_param.upcase, group: { path: 'new_path' } }
 
             expect(response).not_to have_gitlab_http_status(301)
           end
@@ -485,7 +518,7 @@ describe GroupsController do
           let(:redirect_route) { group.redirect_routes.create(path: 'old-path') }
 
           it 'returns not found' do
-            post :update, id: redirect_route.path, group: { path: 'new_path' }
+            post :update, params: { id: redirect_route.path, group: { path: 'new_path' } }
 
             expect(response).to have_gitlab_http_status(404)
           end
@@ -495,13 +528,13 @@ describe GroupsController do
       context 'for a DELETE request' do
         context 'when requesting the canonical path with different casing' do
           it 'does not 404' do
-            delete :destroy, id: group.to_param.upcase
+            delete :destroy, params: { id: group.to_param.upcase }
 
             expect(response).not_to have_gitlab_http_status(404)
           end
 
           it 'does not redirect to the correct casing' do
-            delete :destroy, id: group.to_param.upcase
+            delete :destroy, params: { id: group.to_param.upcase }
 
             expect(response).not_to have_gitlab_http_status(301)
           end
@@ -511,7 +544,7 @@ describe GroupsController do
           let(:redirect_route) { group.redirect_routes.create(path: 'old-path') }
 
           it 'returns not found' do
-            delete :destroy, id: redirect_route.path
+            delete :destroy, params: { id: redirect_route.path }
 
             expect(response).to have_gitlab_http_status(404)
           end
@@ -536,15 +569,17 @@ describe GroupsController do
 
       before do
         put :transfer,
-          id: group.to_param,
-          new_parent_group_id: new_parent_group.id
+          params: {
+            id: group.to_param,
+            new_parent_group_id: new_parent_group.id
+          }
       end
 
-      it 'should return a notice' do
+      it 'returns a notice' do
         expect(flash[:notice]).to eq("Group '#{group.name}' was successfully transferred.")
       end
 
-      it 'should redirect to the new path' do
+      it 'redirects to the new path' do
         expect(response).to redirect_to("/#{new_parent_group.path}/#{group.path}")
       end
     end
@@ -555,15 +590,17 @@ describe GroupsController do
 
       before do
         put :transfer,
-          id: group.to_param,
-          new_parent_group_id: ''
+          params: {
+            id: group.to_param,
+            new_parent_group_id: ''
+          }
       end
 
-      it 'should return a notice' do
+      it 'returns a notice' do
         expect(flash[:notice]).to eq("Group '#{group.name}' was successfully transferred.")
       end
 
-      it 'should redirect to the new path' do
+      it 'redirects to the new path' do
         expect(response).to redirect_to("/#{group.path}")
       end
     end
@@ -577,16 +614,18 @@ describe GroupsController do
         allow_any_instance_of(::Groups::TransferService).to receive(:proceed_to_transfer).and_raise(Gitlab::UpdatePathError, 'namespace directory cannot be moved')
 
         put :transfer,
-          id: group.to_param,
-          new_parent_group_id: new_parent_group.id
+          params: {
+            id: group.to_param,
+            new_parent_group_id: new_parent_group.id
+          }
       end
 
-      it 'should return an alert' do
+      it 'returns an alert' do
         expect(flash[:alert]).to eq "Transfer failed: namespace directory cannot be moved"
       end
 
-      it 'should redirect to the current path' do
-        expect(response).to render_template(:edit)
+      it 'redirects to the current path' do
+        expect(response).to redirect_to(edit_group_path(group))
       end
     end
 
@@ -597,11 +636,13 @@ describe GroupsController do
 
       before do
         put :transfer,
-          id: group.to_param,
-          new_parent_group_id: new_parent_group.id
+          params: {
+            id: group.to_param,
+            new_parent_group_id: new_parent_group.id
+          }
       end
 
-      it 'should be denied' do
+      it 'is denied' do
         expect(response).to have_gitlab_http_status(404)
       end
     end
@@ -626,4 +667,98 @@ describe GroupsController do
       end
     end
   end
+
+  describe 'external authorization' do
+    before do
+      group.add_owner(user)
+      sign_in(user)
+    end
+
+    context 'with external authorization service enabled' do
+      before do
+        enable_external_authorization_service_check
+      end
+
+      describe 'GET #show' do
+        it 'is successful' do
+          get :show, params: { id: group.to_param }
+
+          expect(response).to have_gitlab_http_status(200)
+        end
+
+        it 'does not allow other formats' do
+          get :show, params: { id: group.to_param }, format: :atom
+
+          expect(response).to have_gitlab_http_status(403)
+        end
+      end
+
+      describe 'GET #edit' do
+        it 'is successful' do
+          get :edit, params: { id: group.to_param }
+
+          expect(response).to have_gitlab_http_status(200)
+        end
+      end
+
+      describe 'GET #new' do
+        it 'is successful' do
+          get :new
+
+          expect(response).to have_gitlab_http_status(200)
+        end
+      end
+
+      describe 'GET #index' do
+        it 'is successful' do
+          get :index
+
+          # Redirects to the dashboard
+          expect(response).to have_gitlab_http_status(302)
+        end
+      end
+
+      describe 'POST #create' do
+        it 'creates a group' do
+          expect do
+            post :create, params: { group: { name: 'a name', path: 'a-name' } }
+          end.to change { Group.count }.by(1)
+        end
+      end
+
+      describe 'PUT #update' do
+        it 'updates a group' do
+          expect do
+            put :update, params: { id: group.to_param, group: { name: 'world' } }
+          end.to change { group.reload.name }
+        end
+      end
+
+      describe 'DELETE #destroy' do
+        it 'deletes the group' do
+          delete :destroy, params: { id: group.to_param }
+
+          expect(response).to have_gitlab_http_status(302)
+        end
+      end
+    end
+
+    describe 'GET #activity' do
+      subject { get :activity, params: { id: group.to_param } }
+
+      it_behaves_like 'disabled when using an external authorization service'
+    end
+
+    describe 'GET #issues' do
+      subject { get :issues, params: { id: group.to_param } }
+
+      it_behaves_like 'disabled when using an external authorization service'
+    end
+
+    describe 'GET #merge_requests' do
+      subject { get :merge_requests, params: { id: group.to_param } }
+
+      it_behaves_like 'disabled when using an external authorization service'
+    end
+  end
 end
