@@ -19,26 +19,17 @@ describe Projects::MergeRequestsController do
   describe 'GET commit_change_content' do
     it 'renders commit_change_content template' do
       get :commit_change_content,
-        namespace_id: project.namespace.to_param,
-        project_id: project,
-        id: merge_request.iid,
+        params: {
+          namespace_id: project.namespace.to_param,
+          project_id: project,
+          id: merge_request.iid
+        },
         format: 'html'
 
       expect(response).to render_template('_commit_change_content')
     end
   end
 
-  shared_examples "loads labels" do |action|
-    it "loads labels into the @labels variable" do
-      get action,
-          namespace_id: project.namespace.to_param,
-          project_id: project,
-          id: merge_request.iid,
-          format: 'html'
-      expect(assigns(:labels)).not_to be_nil
-    end
-  end
-
   describe "GET show" do
     def go(extra_params = {})
       params = {
@@ -47,11 +38,9 @@ describe Projects::MergeRequestsController do
         id: merge_request.iid
       }
 
-      get :show, params.merge(extra_params)
+      get :show, params: params.merge(extra_params)
     end
 
-    it_behaves_like "loads labels", :show
-
     describe 'as html' do
       context 'when diff files were cleaned' do
         render_views
@@ -71,6 +60,8 @@ describe Projects::MergeRequestsController do
       end
 
       it "renders merge request page" do
+        expect(::Gitlab::GitalyClient).to receive(:allow_ref_name_caching).and_call_original
+
         go(format: :html)
 
         expect(response).to be_success
@@ -89,6 +80,7 @@ describe Projects::MergeRequestsController do
 
     context 'when user is setting notes filters' do
       let(:issuable) { merge_request }
+      let(:issuable_parent) { project }
       let!(:discussion_note) { create(:discussion_note_on_merge_request, :system, noteable: issuable, project: project) }
       let!(:discussion_comment) { create(:discussion_note_on_merge_request, noteable: issuable, project: project) }
 
@@ -96,6 +88,10 @@ describe Projects::MergeRequestsController do
     end
 
     describe 'as json' do
+      before do
+        expect(::Gitlab::GitalyClient).to receive(:allow_ref_name_caching).and_call_original
+      end
+
       context 'with basic serializer param' do
         it 'renders basic MR entity as json' do
           go(serializer: 'basic', format: :json)
@@ -153,15 +149,22 @@ describe Projects::MergeRequestsController do
 
     def get_merge_requests(page = nil)
       get :index,
-          namespace_id: project.namespace.to_param,
-          project_id: project,
-          state: 'opened', page: page.to_param
+          params: {
+            namespace_id: project.namespace.to_param,
+            project_id: project,
+            state: 'opened',
+            page: page.to_param
+          }
     end
 
     it_behaves_like "issuables list meta-data", :merge_request
 
+    it_behaves_like 'set sort order from user preference' do
+      let(:sorting_param) { 'updated_asc' }
+    end
+
     context 'when page param' do
-      let(:last_page) { project.merge_requests.page().total_pages }
+      let(:last_page) { project.merge_requests.page.total_pages }
       let!(:merge_request) { create(:merge_request_with_diffs, target_project: project, source_project: project) }
 
       it 'redirects to last_page if page number is larger than number of pages' do
@@ -180,11 +183,13 @@ describe Projects::MergeRequestsController do
       it 'does not redirect to external sites when provided a host field' do
         external_host = "www.example.com"
         get :index,
-          namespace_id: project.namespace.to_param,
-          project_id: project,
-          state: 'opened',
-          page: (last_page + 1).to_param,
-          host: external_host
+          params: {
+            namespace_id: project.namespace.to_param,
+            project_id: project,
+            state: 'opened',
+            page: (last_page + 1).to_param,
+            host: external_host
+          }
 
         expect(response).to redirect_to(namespace_project_merge_requests_path(page: last_page, state: controller.params[:state], scope: controller.params[:scope]))
       end
@@ -225,7 +230,7 @@ describe Projects::MergeRequestsController do
         merge_request: mr_params
       }.merge(additional_params)
 
-      put :update, params
+      put :update, params: params
     end
 
     context 'changing the assignee' do
@@ -256,8 +261,8 @@ describe Projects::MergeRequestsController do
     end
 
     context 'there is no source project' do
-      let(:project)       { create(:project, :repository) }
-      let(:forked_project)  { fork_project_with_submodules(project) }
+      let(:project) { create(:project, :repository) }
+      let(:forked_project) { fork_project_with_submodules(project) }
       let!(:merge_request) { create(:merge_request, source_project: forked_project, source_branch: 'add-submodule-version-bump', target_branch: 'master', target_project: project) }
 
       before do
@@ -290,6 +295,20 @@ describe Projects::MergeRequestsController do
 
       it_behaves_like 'update invalid issuable', MergeRequest
     end
+
+    context 'two merge requests with the same source branch' do
+      it 'does not allow a closed merge request to be reopened if another one is open' do
+        merge_request.close!
+        create(:merge_request, source_project: merge_request.source_project, source_branch: merge_request.source_branch)
+
+        update_merge_request(state_event: 'reopen')
+
+        errors = assigns[:merge_request].errors
+
+        expect(errors[:validate_branches]).to include(/Another open merge request already exists for this source branch/)
+        expect(merge_request.reload).to be_closed
+      end
+    end
   end
 
   describe 'POST merge' do
@@ -308,7 +327,7 @@ describe Projects::MergeRequestsController do
 
       before do
         project.add_reporter(user)
-        xhr :post, :merge, base_params
+        post :merge, params: base_params, xhr: true
       end
 
       it 'returns 404' do
@@ -320,7 +339,7 @@ describe Projects::MergeRequestsController do
       before do
         merge_request.update(title: "WIP: #{merge_request.title}")
 
-        post :merge, base_params
+        post :merge, params: base_params
       end
 
       it 'returns :failed' do
@@ -330,7 +349,7 @@ describe Projects::MergeRequestsController do
 
     context 'when the sha parameter does not match the source SHA' do
       before do
-        post :merge, base_params.merge(sha: 'foo')
+        post :merge, params: base_params.merge(sha: 'foo')
       end
 
       it 'returns :sha_mismatch' do
@@ -341,11 +360,7 @@ describe Projects::MergeRequestsController do
     context 'when the sha parameter matches the source SHA' do
       def merge_with_sha(params = {})
         post_params = base_params.merge(sha: merge_request.diff_head_sha).merge(params)
-        if Gitlab.rails5?
-          post :merge, params: post_params, as: :json
-        else
-          post :merge, post_params
-        end
+        post :merge, params: post_params, as: :json
       end
 
       it 'returns :success' do
@@ -378,13 +393,30 @@ describe Projects::MergeRequestsController do
         end
       end
 
+      context 'when a squash commit message is passed' do
+        let(:message) { 'My custom squash commit message' }
+
+        it 'passes the same message to SquashService' do
+          params = { squash: '1', squash_commit_message: message }
+
+          expect_next_instance_of(MergeRequests::SquashService, project, user, params.merge(merge_request: merge_request)) do |squash_service|
+            expect(squash_service).to receive(:execute).and_return({
+              status: :success,
+              squash_sha: SecureRandom.hex(20)
+            })
+          end
+
+          merge_with_sha(params)
+        end
+      end
+
       context 'when the pipeline succeeds is passed' do
         let!(:head_pipeline) do
           create(:ci_empty_pipeline, project: project, sha: merge_request.diff_head_sha, ref: merge_request.source_branch, head_pipeline_of: merge_request)
         end
 
         def merge_when_pipeline_succeeds
-          post :merge, base_params.merge(sha: merge_request.diff_head_sha, merge_when_pipeline_succeeds: '1')
+          post :merge, params: base_params.merge(sha: merge_request.diff_head_sha, merge_when_pipeline_succeeds: '1')
         end
 
         it 'returns :merge_when_pipeline_succeeds' do
@@ -501,7 +533,7 @@ describe Projects::MergeRequestsController do
     let(:user) { create(:user) }
 
     it "denies access to users unless they're admin or project owner" do
-      delete :destroy, namespace_id: project.namespace, project_id: project, id: merge_request.iid
+      delete :destroy, params: { namespace_id: project.namespace, project_id: project, id: merge_request.iid }
 
       expect(response).to have_gitlab_http_status(404)
     end
@@ -516,7 +548,7 @@ describe Projects::MergeRequestsController do
       end
 
       it "deletes the merge request" do
-        delete :destroy, namespace_id: project.namespace, project_id: project, id: merge_request.iid
+        delete :destroy, params: { namespace_id: project.namespace, project_id: project, id: merge_request.iid }
 
         expect(response).to have_gitlab_http_status(302)
         expect(controller).to set_flash[:notice].to(/The merge request was successfully deleted\./)
@@ -525,7 +557,7 @@ describe Projects::MergeRequestsController do
       it 'delegates the update of the todos count cache to TodoService' do
         expect_any_instance_of(TodoService).to receive(:destroy_target).with(merge_request).once
 
-        delete :destroy, namespace_id: project.namespace, project_id: project, id: merge_request.iid
+        delete :destroy, params: { namespace_id: project.namespace, project_id: project, id: merge_request.iid }
       end
     end
   end
@@ -533,9 +565,11 @@ describe Projects::MergeRequestsController do
   describe 'GET commits' do
     def go(format: 'html')
       get :commits,
-          namespace_id: project.namespace.to_param,
-          project_id: project,
-          id: merge_request.iid,
+          params: {
+            namespace_id: project.namespace.to_param,
+            project_id: project,
+            id: merge_request.iid
+          },
           format: format
     end
 
@@ -554,9 +588,11 @@ describe Projects::MergeRequestsController do
                            sha: merge_request.diff_head_sha)
 
       get :pipelines,
-          namespace_id: project.namespace.to_param,
-          project_id: project,
-          id: merge_request.iid,
+          params: {
+            namespace_id: project.namespace.to_param,
+            project_id: project,
+            id: merge_request.iid
+          },
           format: :json
     end
 
@@ -570,9 +606,11 @@ describe Projects::MergeRequestsController do
   describe 'GET test_reports' do
     subject do
       get :test_reports,
-          namespace_id: project.namespace.to_param,
-          project_id: project,
-          id: merge_request.iid,
+          params: {
+            namespace_id: project.namespace.to_param,
+            project_id: project,
+            id: merge_request.iid
+          },
           format: :json
     end
 
@@ -654,11 +692,14 @@ describe Projects::MergeRequestsController do
       merge_request.title = merge_request.wip_title
       merge_request.save
 
-      xhr :post, :remove_wip,
-        namespace_id: merge_request.project.namespace.to_param,
-        project_id: merge_request.project,
-        id: merge_request.iid,
-        format: :json
+      post :remove_wip,
+        params: {
+          format: :json,
+          namespace_id: merge_request.project.namespace.to_param,
+          project_id: merge_request.project,
+          id: merge_request.iid
+        },
+        xhr: true
     end
 
     it 'removes the wip status' do
@@ -672,11 +713,14 @@ describe Projects::MergeRequestsController do
 
   describe 'POST cancel_merge_when_pipeline_succeeds' do
     subject do
-      xhr :post, :cancel_merge_when_pipeline_succeeds,
-        namespace_id: merge_request.project.namespace.to_param,
-        project_id: merge_request.project,
-        id: merge_request.iid,
-        format: :json
+      post :cancel_merge_when_pipeline_succeeds,
+        params: {
+          format: :json,
+          namespace_id: merge_request.project.namespace.to_param,
+          project_id: merge_request.project,
+          id: merge_request.iid
+        },
+        xhr: true
     end
 
     it 'calls MergeRequests::MergeWhenPipelineSucceedsService' do
@@ -711,9 +755,11 @@ describe Projects::MergeRequestsController do
                             target_branch: 'master')
 
       post :assign_related_issues,
-           namespace_id: project.namespace.to_param,
-           project_id: project,
-           id: merge_request.iid
+           params: {
+             namespace_id: project.namespace.to_param,
+             project_id: project,
+             id: merge_request.iid
+           }
     end
 
     it 'shows a flash message on success' do
@@ -812,7 +858,7 @@ describe Projects::MergeRequestsController do
           format: 'json'
         }
 
-        get :ci_environments_status, params.merge(extra_params)
+        get :ci_environments_status, params: params.merge(extra_params)
       end
     end
   end
@@ -853,18 +899,20 @@ describe Projects::MergeRequestsController do
     end
 
     def get_pipeline_status
-      get :pipeline_status, namespace_id: project.namespace,
-                            project_id: project,
-                            id: merge_request.iid,
+      get :pipeline_status, params: {
+                              namespace_id: project.namespace,
+                              project_id: project,
+                              id: merge_request.iid
+                            },
                             format: :json
     end
   end
 
   describe 'POST #rebase' do
-    let(:viewer)        { user }
+    let(:viewer) { user }
 
     def post_rebase
-      post :rebase, namespace_id: project.namespace, project_id: project, id: merge_request
+      post :rebase, params: { namespace_id: project.namespace, project_id: project, id: merge_request }
     end
 
     def expect_rebase_worker_for(user)
@@ -920,15 +968,79 @@ describe Projects::MergeRequestsController do
     end
   end
 
+  describe 'GET discussions' do
+    context 'when authenticated' do
+      before do
+        project.add_developer(user)
+        sign_in(user)
+      end
+
+      it 'returns 200' do
+        get :discussions, params: { namespace_id: project.namespace, project_id: project, id: merge_request.iid }
+
+        expect(response.status).to eq(200)
+      end
+
+      context 'highlight preloading' do
+        context 'with commit diff notes' do
+          let!(:commit_diff_note) do
+            create(:diff_note_on_commit, project: merge_request.project)
+          end
+
+          it 'preloads notes diffs highlights' do
+            expect_next_instance_of(Gitlab::DiscussionsDiff::FileCollection) do |collection|
+              note_diff_file = commit_diff_note.note_diff_file
+
+              expect(collection).to receive(:load_highlight).with([note_diff_file.id]).and_call_original
+              expect(collection).to receive(:find_by_id).with(note_diff_file.id).and_call_original
+            end
+
+            get :discussions, params: { namespace_id: project.namespace, project_id: project, id: merge_request.iid }
+          end
+        end
+
+        context 'with diff notes' do
+          let!(:diff_note) do
+            create(:diff_note_on_merge_request, noteable: merge_request, project: merge_request.project)
+          end
+
+          it 'preloads notes diffs highlights' do
+            expect_next_instance_of(Gitlab::DiscussionsDiff::FileCollection) do |collection|
+              note_diff_file = diff_note.note_diff_file
+
+              expect(collection).to receive(:load_highlight).with([note_diff_file.id]).and_call_original
+              expect(collection).to receive(:find_by_id).with(note_diff_file.id).and_call_original
+            end
+
+            get :discussions, params: { namespace_id: project.namespace, project_id: project, id: merge_request.iid }
+          end
+
+          it 'does not preload highlights when diff note is resolved' do
+            Notes::ResolveService.new(diff_note.project, user).execute(diff_note)
+
+            expect_next_instance_of(Gitlab::DiscussionsDiff::FileCollection) do |collection|
+              note_diff_file = diff_note.note_diff_file
+
+              expect(collection).to receive(:load_highlight).with([]).and_call_original
+              expect(collection).to receive(:find_by_id).with(note_diff_file.id).and_call_original
+            end
+
+            get :discussions, params: { namespace_id: project.namespace, project_id: project, id: merge_request.iid }
+          end
+        end
+      end
+    end
+  end
+
   describe 'GET edit' do
     it 'responds successfully' do
-      get :edit, namespace_id: project.namespace, project_id: project, id: merge_request
+      get :edit, params: { namespace_id: project.namespace, project_id: project, id: merge_request }
 
       expect(response).to have_gitlab_http_status(:success)
     end
 
     it 'assigns the noteable to make sure autocompletes work' do
-      get :edit, namespace_id: project.namespace, project_id: project, id: merge_request
+      get :edit, params: { namespace_id: project.namespace, project_id: project, id: merge_request }
 
       expect(assigns(:noteable)).not_to be_nil
     end
