@@ -8,11 +8,13 @@ end
 shared_examples 'content not cached without revalidation and no-store' do
   it 'ensures content will not be cached without revalidation' do
     # Fixed in newer versions of ActivePack, it will only output a single `private`.
-    if Gitlab.rails5?
-      expect(subject['Cache-Control']).to eq('max-age=0, private, must-revalidate, no-store')
-    else
-      expect(subject['Cache-Control']).to eq('max-age=0, private, must-revalidate, private, no-store')
-    end
+    expect(subject['Cache-Control']).to eq('max-age=0, private, must-revalidate, no-store')
+  end
+end
+
+shared_examples 'content publicly cached' do
+  it 'ensures content is publicly cached' do
+    expect(subject['Cache-Control']).to eq('max-age=300, public')
   end
 end
 
@@ -20,121 +22,160 @@ describe UploadsController do
   let!(:user) { create(:user, avatar: fixture_file_upload("spec/fixtures/dk.png", "image/png")) }
 
   describe 'POST create' do
-    let(:model)   { 'personal_snippet' }
-    let(:snippet) { create(:personal_snippet, :public) }
     let(:jpg)     { fixture_file_upload('spec/fixtures/rails_sample.jpg', 'image/jpg') }
     let(:txt)     { fixture_file_upload('spec/fixtures/doc_sample.txt', 'text/plain') }
 
-    context 'when a user does not have permissions to upload a file' do
-      it "returns 401 when the user is not logged in" do
-        post :create, model: model, id: snippet.id, format: :json
+    context 'snippet uploads' do
+      let(:model)   { 'personal_snippet' }
+      let(:snippet) { create(:personal_snippet, :public) }
+
+      context 'when a user does not have permissions to upload a file' do
+        it "returns 401 when the user is not logged in" do
+          post :create, params: { model: model, id: snippet.id }, format: :json
+
+          expect(response).to have_gitlab_http_status(401)
+        end
+
+        it "returns 404 when user can't comment on a snippet" do
+          private_snippet = create(:personal_snippet, :private)
+
+          sign_in(user)
+          post :create, params: { model: model, id: private_snippet.id }, format: :json
+
+          expect(response).to have_gitlab_http_status(404)
+        end
+      end
+
+      context 'when a user is logged in' do
+        before do
+          sign_in(user)
+        end
+
+        it "returns an error without file" do
+          post :create, params: { model: model, id: snippet.id }, format: :json
+
+          expect(response).to have_gitlab_http_status(422)
+        end
+
+        it "returns an error with invalid model" do
+          expect { post :create, params: { model: 'invalid', id: snippet.id }, format: :json }
+            .to raise_error(ActionController::UrlGenerationError)
+        end
+
+        it "returns 404 status when object not found" do
+          post :create, params: { model: model, id: 9999 }, format: :json
+
+          expect(response).to have_gitlab_http_status(404)
+        end
+
+        context 'with valid image' do
+          before do
+            post :create, params: { model: 'personal_snippet', id: snippet.id, file: jpg }, format: :json
+          end
+
+          it 'returns a content with original filename, new link, and correct type.' do
+            expect(response.body).to match '\"alt\":\"rails_sample\"'
+            expect(response.body).to match "\"url\":\"/uploads"
+          end
+
+          it 'creates a corresponding Upload record' do
+            upload = Upload.last
+
+            aggregate_failures do
+              expect(upload).to exist
+              expect(upload.model).to eq snippet
+            end
+          end
+        end
+
+        context 'with valid non-image file' do
+          before do
+            post :create, params: { model: 'personal_snippet', id: snippet.id, file: txt }, format: :json
+          end
+
+          it 'returns a content with original filename, new link, and correct type.' do
+            expect(response.body).to match '\"alt\":\"doc_sample.txt\"'
+            expect(response.body).to match "\"url\":\"/uploads"
+          end
+
+          it 'creates a corresponding Upload record' do
+            upload = Upload.last
+
+            aggregate_failures do
+              expect(upload).to exist
+              expect(upload.model).to eq snippet
+            end
+          end
+        end
+      end
+    end
+
+    context 'user uploads' do
+      let(:model) { 'user' }
+
+      it 'returns 401 when the user has no access' do
+        post :create, params: { model: 'user', id: user.id }, format: :json
 
         expect(response).to have_gitlab_http_status(401)
       end
 
-      it "returns 404 when user can't comment on a snippet" do
-        private_snippet = create(:personal_snippet, :private)
-
-        sign_in(user)
-        post :create, model: model, id: private_snippet.id, format: :json
-
-        expect(response).to have_gitlab_http_status(404)
-      end
-    end
-
-    context 'when a user is logged in' do
-      before do
-        sign_in(user)
-      end
-
-      it "returns an error without file" do
-        post :create, model: model, id: snippet.id, format: :json
-
-        expect(response).to have_gitlab_http_status(422)
-      end
-
-      it "returns an error with invalid model" do
-        expect { post :create, model: 'invalid', id: snippet.id, format: :json }
-        .to raise_error(ActionController::UrlGenerationError)
-      end
-
-      it "returns 404 status when object not found" do
-        post :create, model: model, id: 9999, format: :json
-
-        expect(response).to have_gitlab_http_status(404)
-      end
-
-      context 'with valid image' do
+      context 'when user is logged in' do
         before do
-          post :create, model: 'personal_snippet', id: snippet.id, file: jpg, format: :json
+          sign_in(user)
         end
 
-        it 'returns a content with original filename, new link, and correct type.' do
-          expect(response.body).to match '\"alt\":\"rails_sample\"'
-          expect(response.body).to match "\"url\":\"/uploads"
-        end
-
-        it 'creates a corresponding Upload record' do
-          upload = Upload.last
-
-          aggregate_failures do
-            expect(upload).to exist
-            expect(upload.model).to eq snippet
-          end
-        end
-      end
-
-      context 'with valid non-image file' do
-        before do
-          post :create, model: 'personal_snippet', id: snippet.id, file: txt, format: :json
-        end
-
-        it 'returns a content with original filename, new link, and correct type.' do
-          expect(response.body).to match '\"alt\":\"doc_sample.txt\"'
-          expect(response.body).to match "\"url\":\"/uploads"
-        end
-
-        it 'creates a corresponding Upload record' do
-          upload = Upload.last
-
-          aggregate_failures do
-            expect(upload).to exist
-            expect(upload.model).to eq snippet
-          end
-        end
-      end
-
-      context 'temporal with valid image' do
         subject do
-          post :create, model: 'personal_snippet', file: jpg, format: :json
+          post :create, params: { model: model, id: user.id, file: jpg }, format: :json
         end
 
         it 'returns a content with original filename, new link, and correct type.' do
           subject
 
           expect(response.body).to match '\"alt\":\"rails_sample\"'
-          expect(response.body).to match "\"url\":\"/uploads/-/system/temp"
+          expect(response.body).to match "\"url\":\"/uploads/-/system/user/#{user.id}/"
         end
 
-        it 'does not create an Upload record' do
-          expect { subject }.not_to change { Upload.count }
-        end
-      end
+        it 'creates a corresponding Upload record' do
+          expect { subject }.to change { Upload.count }
 
-      context 'temporal with valid non-image file' do
-        subject do
-          post :create, model: 'personal_snippet', file: txt, format: :json
+          upload = Upload.last
+
+          aggregate_failures do
+            expect(upload).to exist
+            expect(upload.model).to eq user
+          end
         end
 
-        it 'returns a content with original filename, new link, and correct type.' do
-          subject
+        context 'with valid non-image file' do
+          subject do
+            post :create, params: { model: model, id: user.id, file: txt }, format: :json
+          end
 
-          expect(response.body).to match '\"alt\":\"doc_sample.txt\"'
-          expect(response.body).to match "\"url\":\"/uploads/-/system/temp"
+          it 'returns a content with original filename, new link, and correct type.' do
+            subject
+
+            expect(response.body).to match '\"alt\":\"doc_sample.txt\"'
+            expect(response.body).to match "\"url\":\"/uploads/-/system/user/#{user.id}/"
+          end
+
+          it 'creates a corresponding Upload record' do
+            expect { subject }.to change { Upload.count }
+
+            upload = Upload.last
+
+            aggregate_failures do
+              expect(upload).to exist
+              expect(upload.model).to eq user
+            end
+          end
         end
 
-        it 'does not create an Upload record' do
-          expect { subject }.not_to change { Upload.count }
+        it 'returns 404 when given user is not the logged in one' do
+          another_user = create(:user)
+
+          post :create, params: { model: model, id: another_user.id, file: txt }, format: :json
+
+          expect(response).to have_gitlab_http_status(404)
         end
       end
     end
@@ -147,7 +188,7 @@ describe UploadsController do
       context 'for PNG files' do
         it 'returns Content-Disposition: inline' do
           note = create(:note, :with_attachment, project: project)
-          get :show, model: 'note', mounted_as: 'attachment', id: note.id, filename: 'dk.png'
+          get :show, params: { model: 'note', mounted_as: 'attachment', id: note.id, filename: 'dk.png' }
 
           expect(response['Content-Disposition']).to start_with('inline;')
         end
@@ -156,7 +197,7 @@ describe UploadsController do
       context 'for SVG files' do
         it 'returns Content-Disposition: attachment' do
           note = create(:note, :with_svg_attachment, project: project)
-          get :show, model: 'note', mounted_as: 'attachment', id: note.id, filename: 'unsanitized.svg'
+          get :show, params: { model: 'note', mounted_as: 'attachment', id: note.id, filename: 'unsanitized.svg' }
 
           expect(response['Content-Disposition']).to start_with('attachment;')
         end
@@ -175,7 +216,7 @@ describe UploadsController do
           end
 
           it "redirects to the sign in page" do
-            get :show, model: "user", mounted_as: "avatar", id: user.id, filename: "dk.png"
+            get :show, params: { model: "user", mounted_as: "avatar", id: user.id, filename: "dk.png" }
 
             expect(response).to redirect_to(new_user_session_path)
           end
@@ -183,14 +224,14 @@ describe UploadsController do
 
         context "when the user isn't blocked" do
           it "responds with status 200" do
-            get :show, model: "user", mounted_as: "avatar", id: user.id, filename: "dk.png"
+            get :show, params: { model: "user", mounted_as: "avatar", id: user.id, filename: "dk.png" }
 
             expect(response).to have_gitlab_http_status(200)
           end
 
-          it_behaves_like 'content not cached without revalidation and no-store' do
+          it_behaves_like 'content publicly cached' do
             subject do
-              get :show, model: 'user', mounted_as: 'avatar', id: user.id, filename: 'dk.png'
+              get :show, params: { model: 'user', mounted_as: 'avatar', id: user.id, filename: 'dk.png' }
 
               response
             end
@@ -200,14 +241,14 @@ describe UploadsController do
 
       context "when not signed in" do
         it "responds with status 200" do
-          get :show, model: "user", mounted_as: "avatar", id: user.id, filename: "dk.png"
+          get :show, params: { model: "user", mounted_as: "avatar", id: user.id, filename: "dk.png" }
 
           expect(response).to have_gitlab_http_status(200)
         end
 
-        it_behaves_like 'content not cached without revalidation' do
+        it_behaves_like 'content publicly cached' do
           subject do
-            get :show, model: 'user', mounted_as: 'avatar', id: user.id, filename: 'dk.png'
+            get :show, params: { model: 'user', mounted_as: 'avatar', id: user.id, filename: 'dk.png' }
 
             response
           end
@@ -225,14 +266,14 @@ describe UploadsController do
 
         context "when not signed in" do
           it "responds with status 200" do
-            get :show, model: "project", mounted_as: "avatar", id: project.id, filename: "dk.png"
+            get :show, params: { model: "project", mounted_as: "avatar", id: project.id, filename: "dk.png" }
 
             expect(response).to have_gitlab_http_status(200)
           end
 
           it_behaves_like 'content not cached without revalidation' do
             subject do
-              get :show, model: 'project', mounted_as: 'avatar', id: project.id, filename: 'dk.png'
+              get :show, params: { model: 'project', mounted_as: 'avatar', id: project.id, filename: 'dk.png' }
 
               response
             end
@@ -245,14 +286,14 @@ describe UploadsController do
           end
 
           it "responds with status 200" do
-            get :show, model: "project", mounted_as: "avatar", id: project.id, filename: "dk.png"
+            get :show, params: { model: "project", mounted_as: "avatar", id: project.id, filename: "dk.png" }
 
             expect(response).to have_gitlab_http_status(200)
           end
 
           it_behaves_like 'content not cached without revalidation and no-store' do
             subject do
-              get :show, model: 'project', mounted_as: 'avatar', id: project.id, filename: 'dk.png'
+              get :show, params: { model: 'project', mounted_as: 'avatar', id: project.id, filename: 'dk.png' }
 
               response
             end
@@ -267,7 +308,7 @@ describe UploadsController do
 
         context "when not signed in" do
           it "redirects to the sign in page" do
-            get :show, model: "project", mounted_as: "avatar", id: project.id, filename: "dk.png"
+            get :show, params: { model: "project", mounted_as: "avatar", id: project.id, filename: "dk.png" }
 
             expect(response).to redirect_to(new_user_session_path)
           end
@@ -290,7 +331,7 @@ describe UploadsController do
               end
 
               it "redirects to the sign in page" do
-                get :show, model: "project", mounted_as: "avatar", id: project.id, filename: "dk.png"
+                get :show, params: { model: "project", mounted_as: "avatar", id: project.id, filename: "dk.png" }
 
                 expect(response).to redirect_to(new_user_session_path)
               end
@@ -298,14 +339,14 @@ describe UploadsController do
 
             context "when the user isn't blocked" do
               it "responds with status 200" do
-                get :show, model: "project", mounted_as: "avatar", id: project.id, filename: "dk.png"
+                get :show, params: { model: "project", mounted_as: "avatar", id: project.id, filename: "dk.png" }
 
                 expect(response).to have_gitlab_http_status(200)
               end
 
               it_behaves_like 'content not cached without revalidation and no-store' do
                 subject do
-                  get :show, model: 'project', mounted_as: 'avatar', id: project.id, filename: 'dk.png'
+                  get :show, params: { model: 'project', mounted_as: 'avatar', id: project.id, filename: 'dk.png' }
 
                   response
                 end
@@ -315,7 +356,7 @@ describe UploadsController do
 
           context "when the user doesn't have access to the project" do
             it "responds with status 404" do
-              get :show, model: "project", mounted_as: "avatar", id: project.id, filename: "dk.png"
+              get :show, params: { model: "project", mounted_as: "avatar", id: project.id, filename: "dk.png" }
 
               expect(response).to have_gitlab_http_status(404)
             end
@@ -325,19 +366,19 @@ describe UploadsController do
     end
 
     context "when viewing a group avatar" do
-      let!(:group)   { create(:group, avatar: fixture_file_upload("spec/fixtures/dk.png", "image/png")) }
+      let!(:group) { create(:group, avatar: fixture_file_upload("spec/fixtures/dk.png", "image/png")) }
 
       context "when the group is public" do
         context "when not signed in" do
           it "responds with status 200" do
-            get :show, model: "group", mounted_as: "avatar", id: group.id, filename: "dk.png"
+            get :show, params: { model: "group", mounted_as: "avatar", id: group.id, filename: "dk.png" }
 
             expect(response).to have_gitlab_http_status(200)
           end
 
           it_behaves_like 'content not cached without revalidation' do
             subject do
-              get :show, model: 'group', mounted_as: 'avatar', id: group.id, filename: 'dk.png'
+              get :show, params: { model: 'group', mounted_as: 'avatar', id: group.id, filename: 'dk.png' }
 
               response
             end
@@ -350,14 +391,14 @@ describe UploadsController do
           end
 
           it "responds with status 200" do
-            get :show, model: "group", mounted_as: "avatar", id: group.id, filename: "dk.png"
+            get :show, params: { model: "group", mounted_as: "avatar", id: group.id, filename: "dk.png" }
 
             expect(response).to have_gitlab_http_status(200)
           end
 
           it_behaves_like 'content not cached without revalidation and no-store' do
             subject do
-              get :show, model: 'group', mounted_as: 'avatar', id: group.id, filename: 'dk.png'
+              get :show, params: { model: 'group', mounted_as: 'avatar', id: group.id, filename: 'dk.png' }
 
               response
             end
@@ -386,7 +427,7 @@ describe UploadsController do
               end
 
               it "redirects to the sign in page" do
-                get :show, model: "group", mounted_as: "avatar", id: group.id, filename: "dk.png"
+                get :show, params: { model: "group", mounted_as: "avatar", id: group.id, filename: "dk.png" }
 
                 expect(response).to redirect_to(new_user_session_path)
               end
@@ -394,14 +435,14 @@ describe UploadsController do
 
             context "when the user isn't blocked" do
               it "responds with status 200" do
-                get :show, model: "group", mounted_as: "avatar", id: group.id, filename: "dk.png"
+                get :show, params: { model: "group", mounted_as: "avatar", id: group.id, filename: "dk.png" }
 
                 expect(response).to have_gitlab_http_status(200)
               end
 
               it_behaves_like 'content not cached without revalidation and no-store' do
                 subject do
-                  get :show, model: 'group', mounted_as: 'avatar', id: group.id, filename: 'dk.png'
+                  get :show, params: { model: 'group', mounted_as: 'avatar', id: group.id, filename: 'dk.png' }
 
                   response
                 end
@@ -411,7 +452,7 @@ describe UploadsController do
 
           context "when the user doesn't have access to the project" do
             it "responds with status 404" do
-              get :show, model: "group", mounted_as: "avatar", id: group.id, filename: "dk.png"
+              get :show, params: { model: "group", mounted_as: "avatar", id: group.id, filename: "dk.png" }
 
               expect(response).to have_gitlab_http_status(404)
             end
@@ -431,14 +472,14 @@ describe UploadsController do
 
         context "when not signed in" do
           it "responds with status 200" do
-            get :show, model: "note", mounted_as: "attachment", id: note.id, filename: "dk.png"
+            get :show, params: { model: "note", mounted_as: "attachment", id: note.id, filename: "dk.png" }
 
             expect(response).to have_gitlab_http_status(200)
           end
 
           it_behaves_like 'content not cached without revalidation' do
             subject do
-              get :show, model: 'note', mounted_as: 'attachment', id: note.id, filename: 'dk.png'
+              get :show, params: { model: 'note', mounted_as: 'attachment', id: note.id, filename: 'dk.png' }
 
               response
             end
@@ -451,14 +492,14 @@ describe UploadsController do
           end
 
           it "responds with status 200" do
-            get :show, model: "note", mounted_as: "attachment", id: note.id, filename: "dk.png"
+            get :show, params: { model: "note", mounted_as: "attachment", id: note.id, filename: "dk.png" }
 
             expect(response).to have_gitlab_http_status(200)
           end
 
           it_behaves_like 'content not cached without revalidation and no-store' do
             subject do
-              get :show, model: 'note', mounted_as: 'attachment', id: note.id, filename: 'dk.png'
+              get :show, params: { model: 'note', mounted_as: 'attachment', id: note.id, filename: 'dk.png' }
 
               response
             end
@@ -473,7 +514,7 @@ describe UploadsController do
 
         context "when not signed in" do
           it "redirects to the sign in page" do
-            get :show, model: "note", mounted_as: "attachment", id: note.id, filename: "dk.png"
+            get :show, params: { model: "note", mounted_as: "attachment", id: note.id, filename: "dk.png" }
 
             expect(response).to redirect_to(new_user_session_path)
           end
@@ -496,7 +537,7 @@ describe UploadsController do
               end
 
               it "redirects to the sign in page" do
-                get :show, model: "note", mounted_as: "attachment", id: note.id, filename: "dk.png"
+                get :show, params: { model: "note", mounted_as: "attachment", id: note.id, filename: "dk.png" }
 
                 expect(response).to redirect_to(new_user_session_path)
               end
@@ -504,14 +545,14 @@ describe UploadsController do
 
             context "when the user isn't blocked" do
               it "responds with status 200" do
-                get :show, model: "note", mounted_as: "attachment", id: note.id, filename: "dk.png"
+                get :show, params: { model: "note", mounted_as: "attachment", id: note.id, filename: "dk.png" }
 
                 expect(response).to have_gitlab_http_status(200)
               end
 
               it_behaves_like 'content not cached without revalidation and no-store' do
                 subject do
-                  get :show, model: 'note', mounted_as: 'attachment', id: note.id, filename: 'dk.png'
+                  get :show, params: { model: 'note', mounted_as: 'attachment', id: note.id, filename: 'dk.png' }
 
                   response
                 end
@@ -521,7 +562,7 @@ describe UploadsController do
 
           context "when the user doesn't have access to the project" do
             it "responds with status 404" do
-              get :show, model: "note", mounted_as: "attachment", id: note.id, filename: "dk.png"
+              get :show, params: { model: "note", mounted_as: "attachment", id: note.id, filename: "dk.png" }
 
               expect(response).to have_gitlab_http_status(404)
             end
@@ -536,14 +577,14 @@ describe UploadsController do
 
         context 'when not signed in' do
           it 'responds with status 200' do
-            get :show, model: 'appearance', mounted_as: 'header_logo', id: appearance.id, filename: 'dk.png'
+            get :show, params: { model: 'appearance', mounted_as: 'header_logo', id: appearance.id, filename: 'dk.png' }
 
             expect(response).to have_gitlab_http_status(200)
           end
 
-          it_behaves_like 'content not cached without revalidation' do
+          it_behaves_like 'content publicly cached' do
             subject do
-              get :show, model: 'appearance', mounted_as: 'header_logo', id: appearance.id, filename: 'dk.png'
+              get :show, params: { model: 'appearance', mounted_as: 'header_logo', id: appearance.id, filename: 'dk.png' }
 
               response
             end
@@ -556,14 +597,14 @@ describe UploadsController do
 
         context 'when not signed in' do
           it 'responds with status 200' do
-            get :show, model: 'appearance', mounted_as: 'logo', id: appearance.id, filename: 'dk.png'
+            get :show, params: { model: 'appearance', mounted_as: 'logo', id: appearance.id, filename: 'dk.png' }
 
             expect(response).to have_gitlab_http_status(200)
           end
 
-          it_behaves_like 'content not cached without revalidation' do
+          it_behaves_like 'content publicly cached' do
             subject do
-              get :show, model: 'appearance', mounted_as: 'logo', id: appearance.id, filename: 'dk.png'
+              get :show, params: { model: 'appearance', mounted_as: 'logo', id: appearance.id, filename: 'dk.png' }
 
               response
             end
@@ -577,7 +618,7 @@ describe UploadsController do
 
       context 'has a valid filename on the original file' do
         it 'successfully returns the file' do
-          get :show, model: 'appearance', mounted_as: 'favicon', id: appearance.id, filename: 'dk.png'
+          get :show, params: { model: 'appearance', mounted_as: 'favicon', id: appearance.id, filename: 'dk.png' }
 
           expect(response).to have_gitlab_http_status(200)
           expect(response.header['Content-Disposition']).to end_with 'filename="dk.png"'
@@ -586,7 +627,7 @@ describe UploadsController do
 
       context 'has an invalid filename on the original file' do
         it 'returns a 404' do
-          get :show, model: 'appearance', mounted_as: 'favicon', id: appearance.id, filename: 'bogus.png'
+          get :show, params: { model: 'appearance', mounted_as: 'favicon', id: appearance.id, filename: 'bogus.png' }
 
           expect(response).to have_gitlab_http_status(404)
         end
