@@ -33,6 +33,10 @@ FactoryBot.define do
       status 'pending'
     end
 
+    trait :preparing do
+      status 'preparing'
+    end
+
     trait :created do
       status 'created'
     end
