@@ -1,18 +1,20 @@
 import DirtySubmitForm from '~/dirty_submit/dirty_submit_form';
-import { setInput, createForm } from './helper';
+import { getInputValue, setInputValue, createForm } from './helper';
 
 function expectToToggleDisableOnDirtyUpdate(submit, input) {
-  const originalValue = input.value;
+  const originalValue = getInputValue(input);
 
   expect(submit.disabled).toBe(true);
 
-  return setInput(input, `${originalValue} changes`)
+  return setInputValue(input, `${originalValue} changes`)
     .then(() => expect(submit.disabled).toBe(false))
-    .then(() => setInput(input, originalValue))
+    .then(() => setInputValue(input, originalValue))
     .then(() => expect(submit.disabled).toBe(true));
 }
 
 describe('DirtySubmitForm', () => {
+  DirtySubmitForm.THROTTLE_DURATION = 0;
+
   it('disables submit until there are changes', done => {
     const { form, input, submit } = createForm();
 
@@ -33,4 +35,24 @@ describe('DirtySubmitForm', () => {
       .then(done)
       .catch(done.fail);
   });
+
+  it('disables submit until there are changes for radio inputs', done => {
+    const { form, input, submit } = createForm('radio');
+
+    new DirtySubmitForm(form); // eslint-disable-line no-new
+
+    return expectToToggleDisableOnDirtyUpdate(submit, input)
+      .then(done)
+      .catch(done.fail);
+  });
+
+  it('disables submit until there are changes for checkbox inputs', done => {
+    const { form, input, submit } = createForm('checkbox');
+
+    new DirtySubmitForm(form); // eslint-disable-line no-new
+
+    return expectToToggleDisableOnDirtyUpdate(submit, input)
+      .then(done)
+      .catch(done.fail);
+  });
 });
