@@ -4,7 +4,7 @@ describe 'Groups (JavaScript fixtures)', type: :controller do
   include JavaScriptFixturesHelpers
 
   let(:admin) { create(:admin) }
-  let(:group) { create(:group, name: 'frontend-fixtures-group' )}
+  let(:group) { create(:group, name: 'frontend-fixtures-group', runners_token: 'runnerstoken:intabulasreferre')}
 
   render_views
 
@@ -18,9 +18,8 @@ describe 'Groups (JavaScript fixtures)', type: :controller do
   end
 
   describe GroupsController, '(JavaScript fixtures)', type: :controller do
-    it 'groups/edit.html.raw' do |example|
-      get :edit,
-        id: group
+    it 'groups/edit.html' do |example|
+      get :edit, params: { id: group }
 
       expect(response).to be_success
       store_frontend_fixture(response, example.description)
@@ -28,9 +27,8 @@ describe 'Groups (JavaScript fixtures)', type: :controller do
   end
 
   describe Groups::Settings::CiCdController, '(JavaScript fixtures)', type: :controller do
-    it 'groups/ci_cd_settings.html.raw' do |example|
-      get :show,
-        group_id: group
+    it 'groups/ci_cd_settings.html' do |example|
+      get :show, params: { group_id: group }
 
       expect(response).to be_success
       store_frontend_fixture(response, example.description)
