@@ -3,7 +3,7 @@ require 'spec_helper'
 context 'U2F' do
   include JavaScriptFixturesHelpers
 
-  let(:user) { create(:user, :two_factor_via_u2f) }
+  let(:user) { create(:user, :two_factor_via_u2f, otp_secret: 'otpsecret:coolkids') }
 
   before(:all) do
     clean_frontend_fixtures('u2f/')
@@ -18,10 +18,10 @@ context 'U2F' do
       set_devise_mapping(context: @request)
     end
 
-    it 'u2f/authenticate.html.raw' do |example|
+    it 'u2f/authenticate.html' do |example|
       allow(controller).to receive(:find_user).and_return(user)
 
-      post :create, user: { login: user.username, password: user.password }
+      post :create, params: { user: { login: user.username, password: user.password } }
 
       expect(response).to be_success
       store_frontend_fixture(response, example.description)
@@ -33,9 +33,10 @@ context 'U2F' do
 
     before do
       sign_in(user)
+      allow_any_instance_of(Profiles::TwoFactorAuthsController).to receive(:build_qr_code).and_return('qrcode:blackandwhitesquares')
     end
 
-    it 'u2f/register.html.raw' do |example|
+    it 'u2f/register.html' do |example|
       get :show
 
       expect(response).to be_success
