@@ -2,6 +2,26 @@ import axios from '~/lib/utils/axios_utils';
 import * as commonUtils from '~/lib/utils/common_utils';
 import MockAdapter from 'axios-mock-adapter';
 import { faviconDataUrl, overlayDataUrl, faviconWithOverlayDataUrl } from './mock_data';
+import breakpointInstance from '~/breakpoints';
+
+const PIXEL_TOLERANCE = 0.2;
+
+/**
+ * Loads a data URL as the src of an
+ * {@link https://developer.mozilla.org/en-US/docs/Web/API/HTMLImageElement/Image|Image}
+ * and resolves to that Image once loaded.
+ *
+ * @param url
+ * @returns {Promise}
+ */
+const urlToImage = url =>
+  new Promise(resolve => {
+    const img = new Image();
+    img.onload = function() {
+      resolve(img);
+    };
+    img.src = url;
+  });
 
 describe('common_utils', () => {
   describe('parseUrl', () => {
@@ -213,6 +233,21 @@ describe('common_utils', () => {
     });
   });
 
+  describe('debounceByAnimationFrame', () => {
+    it('debounces a function to allow a maximum of one call per animation frame', done => {
+      const spy = jasmine.createSpy('spy');
+      const debouncedSpy = commonUtils.debounceByAnimationFrame(spy);
+      window.requestAnimationFrame(() => {
+        debouncedSpy();
+        debouncedSpy();
+        window.requestAnimationFrame(() => {
+          expect(spy).toHaveBeenCalledTimes(1);
+          done();
+        });
+      });
+    });
+  });
+
   describe('getParameterByName', () => {
     beforeEach(() => {
       window.history.pushState({}, null, '?scope=all&p=2');
@@ -346,10 +381,64 @@ describe('common_utils', () => {
     });
   });
 
-  describe('convertPermissionToBoolean', () => {
-    it('should convert a boolean in a string to a boolean', () => {
-      expect(commonUtils.convertPermissionToBoolean('true')).toEqual(true);
-      expect(commonUtils.convertPermissionToBoolean('false')).toEqual(false);
+  describe('contentTop', () => {
+    it('does not add height for fileTitle or compareVersionsHeader if screen is too small', () => {
+      spyOn(breakpointInstance, 'isDesktop').and.returnValue(false);
+
+      setFixtures(`
+        <div class="diff-file file-title-flex-parent">
+          blah blah blah
+        </div>
+        <div class="mr-version-controls">
+          more blah blah blah
+        </div>
+      `);
+
+      expect(commonUtils.contentTop()).toBe(0);
+    });
+
+    it('adds height for fileTitle and compareVersionsHeader screen is large enough', () => {
+      spyOn(breakpointInstance, 'isDesktop').and.returnValue(true);
+
+      setFixtures(`
+        <div class="diff-file file-title-flex-parent">
+          blah blah blah
+        </div>
+        <div class="mr-version-controls">
+          more blah blah blah
+        </div>
+      `);
+
+      expect(commonUtils.contentTop()).toBe(18);
+    });
+  });
+
+  describe('parseBoolean', () => {
+    const { parseBoolean } = commonUtils;
+
+    it('returns true for "true"', () => {
+      expect(parseBoolean('true')).toEqual(true);
+    });
+
+    it('returns false for "false"', () => {
+      expect(parseBoolean('false')).toEqual(false);
+    });
+
+    it('returns false for "something"', () => {
+      expect(parseBoolean('something')).toEqual(false);
+    });
+
+    it('returns false for null', () => {
+      expect(parseBoolean(null)).toEqual(false);
+    });
+
+    it('is idempotent', () => {
+      const input = ['true', 'false', 'something', null];
+      input.forEach(value => {
+        const result = parseBoolean(value);
+
+        expect(parseBoolean(result)).toBe(result);
+      });
     });
   });
 
@@ -425,14 +514,16 @@ describe('common_utils', () => {
     });
 
     it('rejects the backOff promise after timing out', done => {
-      commonUtils.backOff(next => next(), 64000).catch(errBackoffResp => {
-        const timeouts = window.setTimeout.calls.allArgs().map(([, timeout]) => timeout);
+      commonUtils
+        .backOff(next => next(), 64000)
+        .catch(errBackoffResp => {
+          const timeouts = window.setTimeout.calls.allArgs().map(([, timeout]) => timeout);
 
-        expect(timeouts).toEqual([2000, 4000, 8000, 16000, 32000, 32000]);
-        expect(errBackoffResp instanceof Error).toBe(true);
-        expect(errBackoffResp.message).toBe('BACKOFF_TIMEOUT');
-        done();
-      });
+          expect(timeouts).toEqual([2000, 4000, 8000, 16000, 32000, 32000]);
+          expect(errBackoffResp instanceof Error).toBe(true);
+          expect(errBackoffResp.message).toBe('BACKOFF_TIMEOUT');
+          done();
+        });
     });
   });
 
@@ -482,8 +573,9 @@ describe('common_utils', () => {
     it('should return the favicon with the overlay', done => {
       commonUtils
         .createOverlayIcon(faviconDataUrl, overlayDataUrl)
-        .then(url => {
-          expect(url).toEqual(faviconWithOverlayDataUrl);
+        .then(url => Promise.all([urlToImage(url), urlToImage(faviconWithOverlayDataUrl)]))
+        .then(([actual, expected]) => {
+          expect(actual).toImageDiffEqual(expected, PIXEL_TOLERANCE);
           done();
         })
         .catch(done.fail);
@@ -505,10 +597,10 @@ describe('common_utils', () => {
     it('should set page favicon to provided favicon overlay', done => {
       commonUtils
         .setFaviconOverlay(overlayDataUrl)
-        .then(() => {
-          expect(document.getElementById('favicon').getAttribute('href')).toEqual(
-            faviconWithOverlayDataUrl,
-          );
+        .then(() => document.getElementById('favicon').getAttribute('href'))
+        .then(url => Promise.all([urlToImage(url), urlToImage(faviconWithOverlayDataUrl)]))
+        .then(([actual, expected]) => {
+          expect(actual).toImageDiffEqual(expected, PIXEL_TOLERANCE);
           done();
         })
         .catch(done.fail);
@@ -551,10 +643,10 @@ describe('common_utils', () => {
 
       commonUtils
         .setCiStatusFavicon(BUILD_URL)
-        .then(() => {
-          const favicon = document.getElementById('favicon');
-
-          expect(favicon.getAttribute('href')).toEqual(faviconWithOverlayDataUrl);
+        .then(() => document.getElementById('favicon').getAttribute('href'))
+        .then(url => Promise.all([urlToImage(url), urlToImage(faviconWithOverlayDataUrl)]))
+        .then(([actual, expected]) => {
+          expect(actual).toImageDiffEqual(expected, PIXEL_TOLERANCE);
           done();
         })
         .catch(done.fail);
@@ -629,51 +721,131 @@ describe('common_utils', () => {
       });
     });
 
-    describe('deep: true', () => {
-      it('converts object with child objects', () => {
-        const obj = {
-          snake_key: {
-            child_snake_key: 'value',
-          },
-        };
+    describe('with options', () => {
+      const objWithoutChildren = {
+        project_name: 'GitLab CE',
+        group_name: 'GitLab.org',
+        license_type: 'MIT',
+      };
 
-        expect(commonUtils.convertObjectPropsToCamelCase(obj, { deep: true })).toEqual({
-          snakeKey: {
-            childSnakeKey: 'value',
-          },
+      const objWithChildren = {
+        project_name: 'GitLab CE',
+        group_name: 'GitLab.org',
+        license_type: 'MIT',
+        tech_stack: {
+          backend: 'Ruby',
+          frontend_framework: 'Vue',
+          database: 'PostgreSQL',
+        },
+      };
+
+      describe('when options.deep is true', () => {
+        it('converts object with child objects', () => {
+          const obj = {
+            snake_key: {
+              child_snake_key: 'value',
+            },
+          };
+
+          expect(commonUtils.convertObjectPropsToCamelCase(obj, { deep: true })).toEqual({
+            snakeKey: {
+              childSnakeKey: 'value',
+            },
+          });
         });
-      });
 
-      it('converts array with child objects', () => {
-        const arr = [
-          {
-            child_snake_key: 'value',
-          },
-        ];
-
-        expect(commonUtils.convertObjectPropsToCamelCase(arr, { deep: true })).toEqual([
-          {
-            childSnakeKey: 'value',
-          },
-        ]);
-      });
-
-      it('converts array with child arrays', () => {
-        const arr = [
-          [
+        it('converts array with child objects', () => {
+          const arr = [
             {
               child_snake_key: 'value',
             },
-          ],
-        ];
+          ];
 
-        expect(commonUtils.convertObjectPropsToCamelCase(arr, { deep: true })).toEqual([
-          [
+          expect(commonUtils.convertObjectPropsToCamelCase(arr, { deep: true })).toEqual([
             {
               childSnakeKey: 'value',
             },
-          ],
-        ]);
+          ]);
+        });
+
+        it('converts array with child arrays', () => {
+          const arr = [
+            [
+              {
+                child_snake_key: 'value',
+              },
+            ],
+          ];
+
+          expect(commonUtils.convertObjectPropsToCamelCase(arr, { deep: true })).toEqual([
+            [
+              {
+                childSnakeKey: 'value',
+              },
+            ],
+          ]);
+        });
+      });
+
+      describe('when options.dropKeys is provided', () => {
+        it('discards properties mentioned in `dropKeys` array', () => {
+          expect(
+            commonUtils.convertObjectPropsToCamelCase(objWithoutChildren, {
+              dropKeys: ['group_name'],
+            }),
+          ).toEqual({
+            projectName: 'GitLab CE',
+            licenseType: 'MIT',
+          });
+        });
+
+        it('discards properties mentioned in `dropKeys` array when `deep` is true', () => {
+          expect(
+            commonUtils.convertObjectPropsToCamelCase(objWithChildren, {
+              deep: true,
+              dropKeys: ['group_name', 'database'],
+            }),
+          ).toEqual({
+            projectName: 'GitLab CE',
+            licenseType: 'MIT',
+            techStack: {
+              backend: 'Ruby',
+              frontendFramework: 'Vue',
+            },
+          });
+        });
+      });
+
+      describe('when options.ignoreKeyNames is provided', () => {
+        it('leaves properties mentioned in `ignoreKeyNames` array intact', () => {
+          expect(
+            commonUtils.convertObjectPropsToCamelCase(objWithoutChildren, {
+              ignoreKeyNames: ['group_name'],
+            }),
+          ).toEqual({
+            projectName: 'GitLab CE',
+            licenseType: 'MIT',
+            group_name: 'GitLab.org',
+          });
+        });
+
+        it('leaves properties mentioned in `ignoreKeyNames` array intact when `deep` is true', () => {
+          expect(
+            commonUtils.convertObjectPropsToCamelCase(objWithChildren, {
+              deep: true,
+              ignoreKeyNames: ['group_name', 'frontend_framework'],
+            }),
+          ).toEqual({
+            projectName: 'GitLab CE',
+            group_name: 'GitLab.org',
+            licenseType: 'MIT',
+            techStack: {
+              backend: 'Ruby',
+              frontend_framework: 'Vue',
+              database: 'PostgreSQL',
+            },
+          });
+        });
       });
     });
   });
@@ -696,4 +868,30 @@ describe('common_utils', () => {
       expect(commonUtils.roundOffFloat(34567.14159, -5)).toBe(0);
     });
   });
+
+  describe('isInViewport', () => {
+    let el;
+
+    beforeEach(() => {
+      el = document.createElement('div');
+    });
+
+    afterEach(() => {
+      document.body.removeChild(el);
+    });
+
+    it('returns true when provided `el` is in viewport', () => {
+      el.setAttribute('style', `position: absolute; right: ${window.innerWidth + 0.2};`);
+      document.body.appendChild(el);
+
+      expect(commonUtils.isInViewport(el)).toBe(true);
+    });
+
+    it('returns false when provided `el` is not in viewport', () => {
+      el.setAttribute('style', 'position: absolute; top: -1000px; left: -1000px;');
+      document.body.appendChild(el);
+
+      expect(commonUtils.isInViewport(el)).toBe(false);
+    });
+  });
 });
