@@ -8,6 +8,7 @@ describe Gitlab::Shell do
   let(:gitlab_shell) { described_class.new }
   let(:popen_vars) { { 'GIT_TERMINAL_PROMPT' => ENV['GIT_TERMINAL_PROMPT'] } }
   let(:timeout) { Gitlab.config.gitlab_shell.git_timeout }
+  let(:gitlab_authorized_keys) { double }
 
   before do
     allow(Project).to receive(:find).and_return(project)
@@ -49,13 +50,38 @@ describe Gitlab::Shell do
 
   describe '#add_key' do
     context 'when authorized_keys_enabled is true' do
-      it 'removes trailing garbage' do
-        allow(gitlab_shell).to receive(:gitlab_shell_keys_path).and_return(:gitlab_shell_keys_path)
-        expect(gitlab_shell).to receive(:gitlab_shell_fast_execute).with(
-          [:gitlab_shell_keys_path, 'add-key', 'key-123', 'ssh-rsa foobar']
-        )
+      context 'authorized_keys_file not set' do
+        before do
+          stub_gitlab_shell_setting(authorized_keys_file: nil)
+          allow(gitlab_shell)
+            .to receive(:gitlab_shell_keys_path)
+            .and_return(:gitlab_shell_keys_path)
+        end
 
-        gitlab_shell.add_key('key-123', 'ssh-rsa foobar trailing garbage')
+        it 'calls #gitlab_shell_fast_execute with add-key command' do
+          expect(gitlab_shell)
+            .to receive(:gitlab_shell_fast_execute)
+            .with([
+              :gitlab_shell_keys_path,
+              'add-key',
+              'key-123',
+              'ssh-rsa foobar'
+            ])
+
+          gitlab_shell.add_key('key-123', 'ssh-rsa foobar trailing garbage')
+        end
+      end
+
+      context 'authorized_keys_file set' do
+        it 'calls Gitlab::AuthorizedKeys#add_key with id and key' do
+          expect(Gitlab::AuthorizedKeys).to receive(:new).and_return(gitlab_authorized_keys)
+
+          expect(gitlab_authorized_keys)
+            .to receive(:add_key)
+            .with('key-123', 'ssh-rsa foobar')
+
+          gitlab_shell.add_key('key-123', 'ssh-rsa foobar')
+        end
       end
     end
 
@@ -64,10 +90,24 @@ describe Gitlab::Shell do
         stub_application_setting(authorized_keys_enabled: false)
       end
 
-      it 'does nothing' do
-        expect(gitlab_shell).not_to receive(:gitlab_shell_fast_execute)
+      context 'authorized_keys_file not set' do
+        before do
+          stub_gitlab_shell_setting(authorized_keys_file: nil)
+        end
 
-        gitlab_shell.add_key('key-123', 'ssh-rsa foobar trailing garbage')
+        it 'does nothing' do
+          expect(gitlab_shell).not_to receive(:gitlab_shell_fast_execute)
+
+          gitlab_shell.add_key('key-123', 'ssh-rsa foobar trailing garbage')
+        end
+      end
+
+      context 'authorized_keys_file set' do
+        it 'does nothing' do
+          expect(Gitlab::AuthorizedKeys).not_to receive(:new)
+
+          gitlab_shell.add_key('key-123', 'ssh-rsa foobar trailing garbage')
+        end
       end
     end
 
@@ -76,24 +116,89 @@ describe Gitlab::Shell do
         stub_application_setting(authorized_keys_enabled: nil)
       end
 
-      it 'removes trailing garbage' do
-        allow(gitlab_shell).to receive(:gitlab_shell_keys_path).and_return(:gitlab_shell_keys_path)
-        expect(gitlab_shell).to receive(:gitlab_shell_fast_execute).with(
-          [:gitlab_shell_keys_path, 'add-key', 'key-123', 'ssh-rsa foobar']
-        )
+      context 'authorized_keys_file not set' do
+        before do
+          stub_gitlab_shell_setting(authorized_keys_file: nil)
+          allow(gitlab_shell)
+            .to receive(:gitlab_shell_keys_path)
+            .and_return(:gitlab_shell_keys_path)
+        end
 
-        gitlab_shell.add_key('key-123', 'ssh-rsa foobar trailing garbage')
+        it 'calls #gitlab_shell_fast_execute with add-key command' do
+          expect(gitlab_shell)
+            .to receive(:gitlab_shell_fast_execute)
+            .with([
+              :gitlab_shell_keys_path,
+              'add-key',
+              'key-123',
+              'ssh-rsa foobar'
+            ])
+
+          gitlab_shell.add_key('key-123', 'ssh-rsa foobar trailing garbage')
+        end
+      end
+
+      context 'authorized_keys_file set' do
+        it 'calls Gitlab::AuthorizedKeys#add_key with id and key' do
+          expect(Gitlab::AuthorizedKeys).to receive(:new).and_return(gitlab_authorized_keys)
+
+          expect(gitlab_authorized_keys)
+            .to receive(:add_key)
+            .with('key-123', 'ssh-rsa foobar')
+
+          gitlab_shell.add_key('key-123', 'ssh-rsa foobar')
+        end
       end
     end
   end
 
   describe '#batch_add_keys' do
-    context 'when authorized_keys_enabled is true' do
-      it 'instantiates KeyAdder' do
-        expect_any_instance_of(Gitlab::Shell::KeyAdder).to receive(:add_key).with('key-123', 'ssh-rsa foobar')
+    let(:keys) { [double(shell_id: 'key-123', key: 'ssh-rsa foobar')] }
 
-        gitlab_shell.batch_add_keys do |adder|
-          adder.add_key('key-123', 'ssh-rsa foobar')
+    context 'when authorized_keys_enabled is true' do
+      context 'authorized_keys_file not set' do
+        let(:io) { double }
+
+        before do
+          stub_gitlab_shell_setting(authorized_keys_file: nil)
+        end
+
+        context 'valid keys' do
+          before do
+            allow(gitlab_shell)
+              .to receive(:gitlab_shell_keys_path)
+              .and_return(:gitlab_shell_keys_path)
+          end
+
+          it 'calls gitlab-keys with batch-add-keys command' do
+            expect(IO)
+              .to receive(:popen)
+              .with("gitlab_shell_keys_path batch-add-keys", 'w')
+              .and_yield(io)
+
+            expect(io).to receive(:puts).with("key-123\tssh-rsa foobar")
+            expect(gitlab_shell.batch_add_keys(keys)).to be_truthy
+          end
+        end
+
+        context 'invalid keys' do
+          let(:keys) { [double(shell_id: 'key-123', key: "ssh-rsa A\tSDFA\nSGADG")] }
+
+          it 'catches failure and returns false' do
+            expect(gitlab_shell.batch_add_keys(keys)).to be_falsey
+          end
+        end
+      end
+
+      context 'authorized_keys_file set' do
+        it 'calls Gitlab::AuthorizedKeys#batch_add_keys with keys to be added' do
+          expect(Gitlab::AuthorizedKeys).to receive(:new).and_return(gitlab_authorized_keys)
+
+          expect(gitlab_authorized_keys)
+            .to receive(:batch_add_keys)
+            .with(keys)
+
+          gitlab_shell.batch_add_keys(keys)
         end
       end
     end
@@ -103,11 +208,23 @@ describe Gitlab::Shell do
         stub_application_setting(authorized_keys_enabled: false)
       end
 
-      it 'does nothing' do
-        expect_any_instance_of(Gitlab::Shell::KeyAdder).not_to receive(:add_key)
+      context 'authorized_keys_file not set' do
+        before do
+          stub_gitlab_shell_setting(authorized_keys_file: nil)
+        end
 
-        gitlab_shell.batch_add_keys do |adder|
-          adder.add_key('key-123', 'ssh-rsa foobar')
+        it 'does nothing' do
+          expect(IO).not_to receive(:popen)
+
+          gitlab_shell.batch_add_keys(keys)
+        end
+      end
+
+      context 'authorized_keys_file set' do
+        it 'does nothing' do
+          expect(Gitlab::AuthorizedKeys).not_to receive(:new)
+
+          gitlab_shell.batch_add_keys(keys)
         end
       end
     end
@@ -117,11 +234,37 @@ describe Gitlab::Shell do
         stub_application_setting(authorized_keys_enabled: nil)
       end
 
-      it 'instantiates KeyAdder' do
-        expect_any_instance_of(Gitlab::Shell::KeyAdder).to receive(:add_key).with('key-123', 'ssh-rsa foobar')
+      context 'authorized_keys_file not set' do
+        let(:io) { double }
 
-        gitlab_shell.batch_add_keys do |adder|
-          adder.add_key('key-123', 'ssh-rsa foobar')
+        before do
+          stub_gitlab_shell_setting(authorized_keys_file: nil)
+          allow(gitlab_shell)
+            .to receive(:gitlab_shell_keys_path)
+            .and_return(:gitlab_shell_keys_path)
+        end
+
+        it 'calls gitlab-keys with batch-add-keys command' do
+          expect(IO)
+            .to receive(:popen)
+            .with("gitlab_shell_keys_path batch-add-keys", 'w')
+            .and_yield(io)
+
+          expect(io).to receive(:puts).with("key-123\tssh-rsa foobar")
+
+          gitlab_shell.batch_add_keys(keys)
+        end
+      end
+
+      context 'authorized_keys_file set' do
+        it 'calls Gitlab::AuthorizedKeys#batch_add_keys with keys to be added' do
+          expect(Gitlab::AuthorizedKeys).to receive(:new).and_return(gitlab_authorized_keys)
+
+          expect(gitlab_authorized_keys)
+            .to receive(:batch_add_keys)
+            .with(keys)
+
+          gitlab_shell.batch_add_keys(keys)
         end
       end
     end
@@ -129,13 +272,34 @@ describe Gitlab::Shell do
 
   describe '#remove_key' do
     context 'when authorized_keys_enabled is true' do
-      it 'removes trailing garbage' do
-        allow(gitlab_shell).to receive(:gitlab_shell_keys_path).and_return(:gitlab_shell_keys_path)
-        expect(gitlab_shell).to receive(:gitlab_shell_fast_execute).with(
-          [:gitlab_shell_keys_path, 'rm-key', 'key-123', 'ssh-rsa foobar']
-        )
+      context 'authorized_keys_file not set' do
+        before do
+          stub_gitlab_shell_setting(authorized_keys_file: nil)
+          allow(gitlab_shell)
+            .to receive(:gitlab_shell_keys_path)
+            .and_return(:gitlab_shell_keys_path)
+        end
 
-        gitlab_shell.remove_key('key-123', 'ssh-rsa foobar')
+        it 'calls #gitlab_shell_fast_execute with rm-key command' do
+          expect(gitlab_shell)
+            .to receive(:gitlab_shell_fast_execute)
+            .with([
+              :gitlab_shell_keys_path,
+              'rm-key',
+              'key-123'
+            ])
+
+          gitlab_shell.remove_key('key-123')
+        end
+      end
+
+      context 'authorized_keys_file not set' do
+        it 'calls Gitlab::AuthorizedKeys#rm_key with the key to be removed' do
+          expect(Gitlab::AuthorizedKeys).to receive(:new).and_return(gitlab_authorized_keys)
+          expect(gitlab_authorized_keys).to receive(:rm_key).with('key-123')
+
+          gitlab_shell.remove_key('key-123')
+        end
       end
     end
 
@@ -144,10 +308,24 @@ describe Gitlab::Shell do
         stub_application_setting(authorized_keys_enabled: false)
       end
 
-      it 'does nothing' do
-        expect(gitlab_shell).not_to receive(:gitlab_shell_fast_execute)
+      context 'authorized_keys_file not set' do
+        before do
+          stub_gitlab_shell_setting(authorized_keys_file: nil)
+        end
 
-        gitlab_shell.remove_key('key-123', 'ssh-rsa foobar')
+        it 'does nothing' do
+          expect(gitlab_shell).not_to receive(:gitlab_shell_fast_execute)
+
+          gitlab_shell.remove_key('key-123')
+        end
+      end
+
+      context 'authorized_keys_file set' do
+        it 'does nothing' do
+          expect(Gitlab::AuthorizedKeys).not_to receive(:new)
+
+          gitlab_shell.remove_key('key-123')
+        end
       end
     end
 
@@ -156,35 +334,64 @@ describe Gitlab::Shell do
         stub_application_setting(authorized_keys_enabled: nil)
       end
 
-      it 'removes trailing garbage' do
-        allow(gitlab_shell).to receive(:gitlab_shell_keys_path).and_return(:gitlab_shell_keys_path)
-        expect(gitlab_shell).to receive(:gitlab_shell_fast_execute).with(
-          [:gitlab_shell_keys_path, 'rm-key', 'key-123', 'ssh-rsa foobar']
-        )
+      context 'authorized_keys_file not set' do
+        before do
+          stub_gitlab_shell_setting(authorized_keys_file: nil)
+          allow(gitlab_shell)
+            .to receive(:gitlab_shell_keys_path)
+            .and_return(:gitlab_shell_keys_path)
+        end
 
-        gitlab_shell.remove_key('key-123', 'ssh-rsa foobar')
+        it 'calls #gitlab_shell_fast_execute with rm-key command' do
+          expect(gitlab_shell)
+            .to receive(:gitlab_shell_fast_execute)
+            .with([
+              :gitlab_shell_keys_path,
+              'rm-key',
+              'key-123'
+            ])
+
+          gitlab_shell.remove_key('key-123')
+        end
       end
-    end
 
-    context 'when key content is not given' do
-      it 'calls rm-key with only one argument' do
-        allow(gitlab_shell).to receive(:gitlab_shell_keys_path).and_return(:gitlab_shell_keys_path)
-        expect(gitlab_shell).to receive(:gitlab_shell_fast_execute).with(
-          [:gitlab_shell_keys_path, 'rm-key', 'key-123']
-        )
+      context 'authorized_keys_file not set' do
+        it 'calls Gitlab::AuthorizedKeys#rm_key with the key to be removed' do
+          expect(Gitlab::AuthorizedKeys).to receive(:new).and_return(gitlab_authorized_keys)
+          expect(gitlab_authorized_keys).to receive(:rm_key).with('key-123')
 
-        gitlab_shell.remove_key('key-123')
+          gitlab_shell.remove_key('key-123')
+        end
       end
     end
   end
 
   describe '#remove_all_keys' do
     context 'when authorized_keys_enabled is true' do
-      it 'removes trailing garbage' do
-        allow(gitlab_shell).to receive(:gitlab_shell_keys_path).and_return(:gitlab_shell_keys_path)
-        expect(gitlab_shell).to receive(:gitlab_shell_fast_execute).with([:gitlab_shell_keys_path, 'clear'])
+      context 'authorized_keys_file not set' do
+        before do
+          stub_gitlab_shell_setting(authorized_keys_file: nil)
+          allow(gitlab_shell)
+            .to receive(:gitlab_shell_keys_path)
+            .and_return(:gitlab_shell_keys_path)
+        end
 
-        gitlab_shell.remove_all_keys
+        it 'calls #gitlab_shell_fast_execute with clear command' do
+          expect(gitlab_shell)
+            .to receive(:gitlab_shell_fast_execute)
+            .with([:gitlab_shell_keys_path, 'clear'])
+
+          gitlab_shell.remove_all_keys
+        end
+      end
+
+      context 'authorized_keys_file set' do
+        it 'calls Gitlab::AuthorizedKeys#clear' do
+          expect(Gitlab::AuthorizedKeys).to receive(:new).and_return(gitlab_authorized_keys)
+          expect(gitlab_authorized_keys).to receive(:clear)
+
+          gitlab_shell.remove_all_keys
+        end
       end
     end
 
@@ -193,10 +400,24 @@ describe Gitlab::Shell do
         stub_application_setting(authorized_keys_enabled: false)
       end
 
-      it 'does nothing' do
-        expect(gitlab_shell).not_to receive(:gitlab_shell_fast_execute)
+      context 'authorized_keys_file not set' do
+        before do
+          stub_gitlab_shell_setting(authorized_keys_file: nil)
+        end
 
-        gitlab_shell.remove_all_keys
+        it 'does nothing' do
+          expect(gitlab_shell).not_to receive(:gitlab_shell_fast_execute)
+
+          gitlab_shell.remove_all_keys
+        end
+      end
+
+      context 'authorized_keys_file set' do
+        it 'does nothing' do
+          expect(Gitlab::AuthorizedKeys).not_to receive(:new)
+
+          gitlab_shell.remove_all_keys
+        end
       end
     end
 
@@ -205,183 +426,164 @@ describe Gitlab::Shell do
         stub_application_setting(authorized_keys_enabled: nil)
       end
 
-      it 'removes trailing garbage' do
-        allow(gitlab_shell).to receive(:gitlab_shell_keys_path).and_return(:gitlab_shell_keys_path)
-        expect(gitlab_shell).to receive(:gitlab_shell_fast_execute).with(
-          [:gitlab_shell_keys_path, 'clear']
-        )
+      context 'authorized_keys_file not set' do
+        before do
+          stub_gitlab_shell_setting(authorized_keys_file: nil)
+          allow(gitlab_shell)
+            .to receive(:gitlab_shell_keys_path)
+            .and_return(:gitlab_shell_keys_path)
+        end
 
-        gitlab_shell.remove_all_keys
+        it 'calls #gitlab_shell_fast_execute with clear command' do
+          expect(gitlab_shell)
+            .to receive(:gitlab_shell_fast_execute)
+            .with([:gitlab_shell_keys_path, 'clear'])
+
+          gitlab_shell.remove_all_keys
+        end
+      end
+
+      context 'authorized_keys_file set' do
+        it 'calls Gitlab::AuthorizedKeys#clear' do
+          expect(Gitlab::AuthorizedKeys).to receive(:new).and_return(gitlab_authorized_keys)
+          expect(gitlab_authorized_keys).to receive(:clear)
+
+          gitlab_shell.remove_all_keys
+        end
       end
     end
   end
 
   describe '#remove_keys_not_found_in_db' do
     context 'when keys are in the file that are not in the DB' do
-      before do
-        gitlab_shell.remove_all_keys
-        gitlab_shell.add_key('key-1234', 'ssh-rsa ASDFASDF')
-        gitlab_shell.add_key('key-9876', 'ssh-rsa ASDFASDF')
-        @another_key = create(:key) # this one IS in the DB
+      context 'authorized_keys_file not set' do
+        before do
+          stub_gitlab_shell_setting(authorized_keys_file: nil)
+          gitlab_shell.remove_all_keys
+          gitlab_shell.add_key('key-1234', 'ssh-rsa ASDFASDF')
+          gitlab_shell.add_key('key-9876', 'ssh-rsa ASDFASDF')
+          @another_key = create(:key) # this one IS in the DB
+        end
+
+        it 'removes the keys' do
+          expect(gitlab_shell).to receive(:remove_key).with('key-1234')
+          expect(gitlab_shell).to receive(:remove_key).with('key-9876')
+          expect(gitlab_shell).not_to receive(:remove_key).with("key-#{@another_key.id}")
+
+          gitlab_shell.remove_keys_not_found_in_db
+        end
       end
 
-      it 'removes the keys' do
-        expect(find_in_authorized_keys_file(1234)).to be_truthy
-        expect(find_in_authorized_keys_file(9876)).to be_truthy
-        expect(find_in_authorized_keys_file(@another_key.id)).to be_truthy
-        gitlab_shell.remove_keys_not_found_in_db
-        expect(find_in_authorized_keys_file(1234)).to be_falsey
-        expect(find_in_authorized_keys_file(9876)).to be_falsey
-        expect(find_in_authorized_keys_file(@another_key.id)).to be_truthy
+      context 'authorized_keys_file set' do
+        before do
+          gitlab_shell.remove_all_keys
+          gitlab_shell.add_key('key-1234', 'ssh-rsa ASDFASDF')
+          gitlab_shell.add_key('key-9876', 'ssh-rsa ASDFASDF')
+          @another_key = create(:key) # this one IS in the DB
+        end
+
+        it 'removes the keys' do
+          expect(gitlab_shell).to receive(:remove_key).with('key-1234')
+          expect(gitlab_shell).to receive(:remove_key).with('key-9876')
+          expect(gitlab_shell).not_to receive(:remove_key).with("key-#{@another_key.id}")
+
+          gitlab_shell.remove_keys_not_found_in_db
+        end
       end
     end
 
     context 'when keys there are duplicate keys in the file that are not in the DB' do
-      before do
-        gitlab_shell.remove_all_keys
-        gitlab_shell.add_key('key-1234', 'ssh-rsa ASDFASDF')
-        gitlab_shell.add_key('key-1234', 'ssh-rsa ASDFASDF')
+      context 'authorized_keys_file not set' do
+        before do
+          stub_gitlab_shell_setting(authorized_keys_file: nil)
+          gitlab_shell.remove_all_keys
+          gitlab_shell.add_key('key-1234', 'ssh-rsa ASDFASDF')
+          gitlab_shell.add_key('key-1234', 'ssh-rsa ASDFASDF')
+        end
+
+        it 'removes the keys' do
+          expect(gitlab_shell).to receive(:remove_key).with('key-1234')
+
+          gitlab_shell.remove_keys_not_found_in_db
+        end
       end
 
-      it 'removes the keys' do
-        expect(find_in_authorized_keys_file(1234)).to be_truthy
-        gitlab_shell.remove_keys_not_found_in_db
-        expect(find_in_authorized_keys_file(1234)).to be_falsey
-      end
+      context 'authorized_keys_file set' do
+        before do
+          gitlab_shell.remove_all_keys
+          gitlab_shell.add_key('key-1234', 'ssh-rsa ASDFASDF')
+          gitlab_shell.add_key('key-1234', 'ssh-rsa ASDFASDF')
+        end
 
-      it 'does not run remove more than once per key (in a batch)' do
-        expect(gitlab_shell).to receive(:remove_key).with('key-1234').once
-        gitlab_shell.remove_keys_not_found_in_db
+        it 'removes the keys' do
+          expect(gitlab_shell).to receive(:remove_key).with('key-1234')
+
+          gitlab_shell.remove_keys_not_found_in_db
+        end
       end
     end
 
     context 'when keys there are duplicate keys in the file that ARE in the DB' do
-      before do
-        gitlab_shell.remove_all_keys
-        @key = create(:key)
-        gitlab_shell.add_key(@key.shell_id, @key.key)
+      context 'authorized_keys_file not set' do
+        before do
+          stub_gitlab_shell_setting(authorized_keys_file: nil)
+          gitlab_shell.remove_all_keys
+          @key = create(:key)
+          gitlab_shell.add_key(@key.shell_id, @key.key)
+        end
+
+        it 'does not remove the key' do
+          expect(gitlab_shell).not_to receive(:remove_key).with("key-#{@key.id}")
+
+          gitlab_shell.remove_keys_not_found_in_db
+        end
       end
 
-      it 'does not remove the key' do
-        gitlab_shell.remove_keys_not_found_in_db
-        expect(find_in_authorized_keys_file(@key.id)).to be_truthy
-      end
+      context 'authorized_keys_file set' do
+        before do
+          gitlab_shell.remove_all_keys
+          @key = create(:key)
+          gitlab_shell.add_key(@key.shell_id, @key.key)
+        end
 
-      it 'does not need to run a SELECT query for that batch, on account of that key' do
-        expect_any_instance_of(ActiveRecord::Relation).not_to receive(:pluck)
-        gitlab_shell.remove_keys_not_found_in_db
+        it 'does not remove the key' do
+          expect(gitlab_shell).not_to receive(:remove_key).with("key-#{@key.id}")
+
+          gitlab_shell.remove_keys_not_found_in_db
+        end
       end
     end
 
     unless ENV['CI'] # Skip in CI, it takes 1 minute
       context 'when the first batch can be skipped, but the next batch has keys that are not in the DB' do
-        before do
-          gitlab_shell.remove_all_keys
-          100.times { |i| create(:key) } # first batch is all in the DB
-          gitlab_shell.add_key('key-1234', 'ssh-rsa ASDFASDF')
-        end
+        context 'authorized_keys_file not set' do
+          before do
+            stub_gitlab_shell_setting(authorized_keys_file: nil)
+            gitlab_shell.remove_all_keys
+            100.times { |i| create(:key) } # first batch is all in the DB
+            gitlab_shell.add_key('key-1234', 'ssh-rsa ASDFASDF')
+          end
 
-        it 'removes the keys not in the DB' do
-          expect(find_in_authorized_keys_file(1234)).to be_truthy
-          gitlab_shell.remove_keys_not_found_in_db
-          expect(find_in_authorized_keys_file(1234)).to be_falsey
-        end
-      end
-    end
-  end
+          it 'removes the keys not in the DB' do
+            expect(gitlab_shell).to receive(:remove_key).with('key-1234')
 
-  describe '#batch_read_key_ids' do
-    context 'when there are keys in the authorized_keys file' do
-      before do
-        gitlab_shell.remove_all_keys
-        (1..4).each do |i|
-          gitlab_shell.add_key("key-#{i}", "ssh-rsa ASDFASDF#{i}")
-        end
-      end
-
-      it 'iterates over the key IDs in the file, in batches' do
-        loop_count = 0
-        first_batch = [1, 2]
-        second_batch = [3, 4]
-
-        gitlab_shell.batch_read_key_ids(batch_size: 2) do |batch|
-          expected = (loop_count == 0 ? first_batch : second_batch)
-          expect(batch).to eq(expected)
-          loop_count += 1
-        end
-      end
-    end
-  end
-
-  describe '#list_key_ids' do
-    context 'when there are keys in the authorized_keys file' do
-      before do
-        gitlab_shell.remove_all_keys
-        (1..4).each do |i|
-          gitlab_shell.add_key("key-#{i}", "ssh-rsa ASDFASDF#{i}")
-        end
-      end
-
-      it 'outputs the key IDs in the file, separated by newlines' do
-        ids = []
-        gitlab_shell.list_key_ids do |io|
-          io.each do |line|
-            ids << line
+            gitlab_shell.remove_keys_not_found_in_db
           end
         end
 
-        expect(ids).to eq(%W{1\n 2\n 3\n 4\n})
-      end
-    end
+        context 'authorized_keys_file set' do
+          before do
+            gitlab_shell.remove_all_keys
+            100.times { |i| create(:key) } # first batch is all in the DB
+            gitlab_shell.add_key('key-1234', 'ssh-rsa ASDFASDF')
+          end
 
-    context 'when there are no keys in the authorized_keys file' do
-      before do
-        gitlab_shell.remove_all_keys
-      end
+          it 'removes the keys not in the DB' do
+            expect(gitlab_shell).to receive(:remove_key).with('key-1234')
 
-      it 'outputs nothing, not even an empty string' do
-        ids = []
-        gitlab_shell.list_key_ids do |io|
-          io.each do |line|
-            ids << line
+            gitlab_shell.remove_keys_not_found_in_db
           end
         end
-
-        expect(ids).to eq([])
-      end
-    end
-  end
-
-  describe Gitlab::Shell::KeyAdder do
-    describe '#add_key' do
-      it 'removes trailing garbage' do
-        io = spy(:io)
-        adder = described_class.new(io)
-
-        adder.add_key('key-42', "ssh-rsa foo bar\tbaz")
-
-        expect(io).to have_received(:puts).with("key-42\tssh-rsa foo")
-      end
-
-      it 'handles multiple spaces in the key' do
-        io = spy(:io)
-        adder = described_class.new(io)
-
-        adder.add_key('key-42', "ssh-rsa  foo")
-
-        expect(io).to have_received(:puts).with("key-42\tssh-rsa foo")
-      end
-
-      it 'raises an exception if the key contains a tab' do
-        expect do
-          described_class.new(StringIO.new).add_key('key-42', "ssh-rsa\tfoobar")
-        end.to raise_error(Gitlab::Shell::Error)
-      end
-
-      it 'raises an exception if the key contains a newline' do
-        expect do
-          described_class.new(StringIO.new).add_key('key-42', "ssh-rsa foobar\nssh-rsa pawned")
-        end.to raise_error(Gitlab::Shell::Error)
       end
     end
   end
@@ -393,7 +595,6 @@ describe Gitlab::Shell do
 
     before do
       allow(Gitlab.config.gitlab_shell).to receive(:path).and_return(gitlab_shell_path)
-      allow(Gitlab.config.gitlab_shell).to receive(:hooks_path).and_return(gitlab_shell_hooks_path)
       allow(Gitlab.config.gitlab_shell).to receive(:git_timeout).and_return(800)
     end
 
@@ -412,7 +613,7 @@ describe Gitlab::Shell do
       end
 
       it 'creates a repository' do
-        expect(gitlab_shell.create_repository(repository_storage, repo_name)).to be_truthy
+        expect(gitlab_shell.create_repository(repository_storage, repo_name, repo_name)).to be_truthy
 
         expect(File.stat(created_path).mode & 0o777).to eq(0o770)
 
@@ -427,7 +628,7 @@ describe Gitlab::Shell do
         # should cause #create_repository to fail.
         FileUtils.touch(created_path)
 
-        expect(gitlab_shell.create_repository(repository_storage, repo_name)).to be_falsy
+        expect(gitlab_shell.create_repository(repository_storage, repo_name, repo_name)).to be_falsy
       end
     end
 
@@ -474,13 +675,10 @@ describe Gitlab::Shell do
     end
 
     describe '#fork_repository' do
+      let(:target_project) { create(:project) }
+
       subject do
-        gitlab_shell.fork_repository(
-          project.repository_storage,
-          project.disk_path,
-          'nfs-file05',
-          'fork/path'
-        )
+        gitlab_shell.fork_repository(project, target_project)
       end
 
       it 'returns true when the command succeeds' do
@@ -505,7 +703,7 @@ describe Gitlab::Shell do
         it 'returns true when the command succeeds' do
           expect_any_instance_of(Gitlab::GitalyClient::RepositoryService).to receive(:import_repository).with(import_url)
 
-          result = gitlab_shell.import_repository(project.repository_storage, project.disk_path, import_url)
+          result = gitlab_shell.import_repository(project.repository_storage, project.disk_path, import_url, project.full_path)
 
           expect(result).to be_truthy
         end
@@ -516,7 +714,7 @@ describe Gitlab::Shell do
           expect_any_instance_of(Gitlab::Shell::GitalyGitlabProjects).to receive(:output) { 'error'}
 
           expect do
-            gitlab_shell.import_repository(project.repository_storage, project.disk_path, import_url)
+            gitlab_shell.import_repository(project.repository_storage, project.disk_path, import_url, project.full_path)
           end.to raise_error(Gitlab::Shell::Error, "error")
         end
       end
@@ -570,12 +768,4 @@ describe Gitlab::Shell do
       end
     end
   end
-
-  def find_in_authorized_keys_file(key_id)
-    gitlab_shell.batch_read_key_ids do |ids|
-      return true if ids.include?(key_id) # rubocop:disable Cop/AvoidReturnFromBlocks
-    end
-
-    false
-  end
 end
