@@ -1,3 +1,5 @@
+# frozen_string_literal: true
+
 require 'spec_helper'
 
 describe Repository do
@@ -38,49 +40,29 @@ describe Repository do
   end
 
   describe '#branch_names_contains' do
-    shared_examples '#branch_names_contains' do
-      set(:project) { create(:project, :repository) }
-      let(:repository) { project.repository }
+    set(:project) { create(:project, :repository) }
+    let(:repository) { project.repository }
 
-      subject { repository.branch_names_contains(sample_commit.id) }
+    subject { repository.branch_names_contains(sample_commit.id) }
 
-      it { is_expected.to include('master') }
-      it { is_expected.not_to include('feature') }
-      it { is_expected.not_to include('fix') }
+    it { is_expected.to include('master') }
+    it { is_expected.not_to include('feature') }
+    it { is_expected.not_to include('fix') }
 
-      describe 'when storage is broken', :broken_storage do
-        it 'should raise a storage error' do
-          expect_to_raise_storage_error do
-            broken_repository.branch_names_contains(sample_commit.id)
-          end
+    describe 'when storage is broken', :broken_storage do
+      it 'raises a storage error' do
+        expect_to_raise_storage_error do
+          broken_repository.branch_names_contains(sample_commit.id)
         end
       end
     end
-
-    context 'when gitaly is enabled' do
-      it_behaves_like '#branch_names_contains'
-    end
-
-    context 'when gitaly is disabled', :skip_gitaly_mock do
-      it_behaves_like '#branch_names_contains'
-    end
   end
 
   describe '#tag_names_contains' do
-    shared_examples '#tag_names_contains' do
-      subject { repository.tag_names_contains(sample_commit.id) }
+    subject { repository.tag_names_contains(sample_commit.id) }
 
-      it { is_expected.to include('v1.1.0') }
-      it { is_expected.not_to include('v1.0.0') }
-    end
-
-    context 'when gitaly is enabled' do
-      it_behaves_like '#tag_names_contains'
-    end
-
-    context 'when gitaly is enabled', :skip_gitaly_mock do
-      it_behaves_like '#tag_names_contains'
-    end
+    it { is_expected.to include('v1.1.0') }
+    it { is_expected.not_to include('v1.0.0') }
   end
 
   describe 'tags_sorted_by' do
@@ -238,61 +220,41 @@ describe Repository do
   end
 
   describe '#last_commit_for_path' do
-    shared_examples 'getting last commit for path' do
-      subject { repository.last_commit_for_path(sample_commit.id, '.gitignore').id }
+    subject { repository.last_commit_for_path(sample_commit.id, '.gitignore').id }
 
-      it { is_expected.to eq('c1acaa58bbcbc3eafe538cb8274ba387047b69f8') }
+    it { is_expected.to eq('c1acaa58bbcbc3eafe538cb8274ba387047b69f8') }
 
-      describe 'when storage is broken', :broken_storage do
-        it 'should raise a storage error' do
-          expect_to_raise_storage_error do
-            broken_repository.last_commit_id_for_path(sample_commit.id, '.gitignore')
-          end
+    describe 'when storage is broken', :broken_storage do
+      it 'raises a storage error' do
+        expect_to_raise_storage_error do
+          broken_repository.last_commit_id_for_path(sample_commit.id, '.gitignore')
         end
       end
     end
-
-    context 'when Gitaly feature last_commit_for_path is enabled' do
-      it_behaves_like 'getting last commit for path'
-    end
-
-    context 'when Gitaly feature last_commit_for_path is disabled', :skip_gitaly_mock do
-      it_behaves_like 'getting last commit for path'
-    end
   end
 
   describe '#last_commit_id_for_path' do
-    shared_examples 'getting last commit ID for path' do
-      subject { repository.last_commit_id_for_path(sample_commit.id, '.gitignore') }
+    subject { repository.last_commit_id_for_path(sample_commit.id, '.gitignore') }
 
-      it "returns last commit id for a given path" do
-        is_expected.to eq('c1acaa58bbcbc3eafe538cb8274ba387047b69f8')
-      end
+    it "returns last commit id for a given path" do
+      is_expected.to eq('c1acaa58bbcbc3eafe538cb8274ba387047b69f8')
+    end
 
-      it "caches last commit id for a given path" do
-        cache = repository.send(:cache)
-        key = "last_commit_id_for_path:#{sample_commit.id}:#{Digest::SHA1.hexdigest('.gitignore')}"
+    it "caches last commit id for a given path" do
+      cache = repository.send(:cache)
+      key = "last_commit_id_for_path:#{sample_commit.id}:#{Digest::SHA1.hexdigest('.gitignore')}"
 
-        expect(cache).to receive(:fetch).with(key).and_return('c1acaa5')
-        is_expected.to eq('c1acaa5')
-      end
+      expect(cache).to receive(:fetch).with(key).and_return('c1acaa5')
+      is_expected.to eq('c1acaa5')
+    end
 
-      describe 'when storage is broken', :broken_storage do
-        it 'should raise a storage error' do
-          expect_to_raise_storage_error do
-            broken_repository.last_commit_for_path(sample_commit.id, '.gitignore').id
-          end
+    describe 'when storage is broken', :broken_storage do
+      it 'raises a storage error' do
+        expect_to_raise_storage_error do
+          broken_repository.last_commit_for_path(sample_commit.id, '.gitignore').id
         end
       end
     end
-
-    context 'when Gitaly feature last_commit_for_path is enabled' do
-      it_behaves_like 'getting last commit ID for path'
-    end
-
-    context 'when Gitaly feature last_commit_for_path is disabled', :skip_gitaly_mock do
-      it_behaves_like 'getting last commit ID for path'
-    end
   end
 
   describe '#commits' do
@@ -374,82 +336,61 @@ describe Repository do
 
   describe '#commits_by' do
     set(:project) { create(:project, :repository) }
+    let(:oids) { TestEnv::BRANCH_SHA.values }
 
-    shared_examples 'batch commits fetching' do
-      let(:oids) { TestEnv::BRANCH_SHA.values }
+    subject { project.repository.commits_by(oids: oids) }
 
-      subject { project.repository.commits_by(oids: oids) }
+    it 'finds each commit' do
+      expect(subject).not_to include(nil)
+      expect(subject.size).to eq(oids.size)
+    end
 
-      it 'finds each commit' do
+    it 'returns only Commit instances' do
+      expect(subject).to all( be_a(Commit) )
+    end
+
+    context 'when some commits are not found ' do
+      let(:oids) do
+        ['deadbeef'] + TestEnv::BRANCH_SHA.values.first(10)
+      end
+
+      it 'returns only found commits' do
         expect(subject).not_to include(nil)
-        expect(subject.size).to eq(oids.size)
-      end
-
-      it 'returns only Commit instances' do
-        expect(subject).to all( be_a(Commit) )
-      end
-
-      context 'when some commits are not found ' do
-        let(:oids) do
-          ['deadbeef'] + TestEnv::BRANCH_SHA.values.first(10)
-        end
-
-        it 'returns only found commits' do
-          expect(subject).not_to include(nil)
-          expect(subject.size).to eq(10)
-        end
-      end
-
-      context 'when no oids are passed' do
-        let(:oids) { [] }
-
-        it 'does not call #batch_by_oid' do
-          expect(Gitlab::Git::Commit).not_to receive(:batch_by_oid)
-
-          subject
-        end
+        expect(subject.size).to eq(10)
       end
     end
 
-    context 'when Gitaly list_commits_by_oid is enabled' do
-      it_behaves_like 'batch commits fetching'
-    end
+    context 'when no oids are passed' do
+      let(:oids) { [] }
 
-    context 'when Gitaly list_commits_by_oid is enabled', :disable_gitaly do
-      it_behaves_like 'batch commits fetching'
+      it 'does not call #batch_by_oid' do
+        expect(Gitlab::Git::Commit).not_to receive(:batch_by_oid)
+
+        subject
+      end
     end
   end
 
   describe '#find_commits_by_message' do
-    shared_examples 'finding commits by message' do
-      it 'returns commits with messages containing a given string' do
-        commit_ids = repository.find_commits_by_message('submodule').map(&:id)
+    it 'returns commits with messages containing a given string' do
+      commit_ids = repository.find_commits_by_message('submodule').map(&:id)
 
-        expect(commit_ids).to include(
-          '5937ac0a7beb003549fc5fd26fc247adbce4a52e',
-          '6f6d7e7ed97bb5f0054f2b1df789b39ca89b6ff9',
-          'cfe32cf61b73a0d5e9f13e774abde7ff789b1660'
-        )
-        expect(commit_ids).not_to include('913c66a37b4a45b9769037c55c2d238bd0942d2e')
-      end
-
-      it 'is case insensitive' do
-        commit_ids = repository.find_commits_by_message('SUBMODULE').map(&:id)
-
-        expect(commit_ids).to include('5937ac0a7beb003549fc5fd26fc247adbce4a52e')
-      end
+      expect(commit_ids).to include(
+        '5937ac0a7beb003549fc5fd26fc247adbce4a52e',
+        '6f6d7e7ed97bb5f0054f2b1df789b39ca89b6ff9',
+        'cfe32cf61b73a0d5e9f13e774abde7ff789b1660'
+      )
+      expect(commit_ids).not_to include('913c66a37b4a45b9769037c55c2d238bd0942d2e')
     end
 
-    context 'when Gitaly commits_by_message feature is enabled' do
-      it_behaves_like 'finding commits by message'
-    end
+    it 'is case insensitive' do
+      commit_ids = repository.find_commits_by_message('SUBMODULE').map(&:id)
 
-    context 'when Gitaly commits_by_message feature is disabled', :skip_gitaly_mock do
-      it_behaves_like 'finding commits by message'
+      expect(commit_ids).to include('5937ac0a7beb003549fc5fd26fc247adbce4a52e')
     end
 
     describe 'when storage is broken', :broken_storage do
-      it 'should raise a storage error' do
+      it 'raises a storage error' do
         expect_to_raise_storage_error { broken_repository.find_commits_by_message('s') }
       end
     end
@@ -785,7 +726,7 @@ describe Repository do
     end
 
     describe 'when storage is broken', :broken_storage do
-      it 'should raise a storage error' do
+      it 'raises a storage error' do
         expect_to_raise_storage_error do
           broken_repository.search_files_by_content('feature', 'master')
         end
@@ -834,7 +775,7 @@ describe Repository do
     end
 
     describe 'when storage is broken', :broken_storage do
-      it 'should raise a storage error' do
+      it 'raises a storage error' do
         expect_to_raise_storage_error { broken_repository.search_files_by_name('files', 'master') }
       end
     end
@@ -876,7 +817,7 @@ describe Repository do
     let(:broken_repository) { create(:project, :broken_storage).repository }
 
     describe 'when storage is broken', :broken_storage do
-      it 'should raise a storage error' do
+      it 'raises a storage error' do
         expect_to_raise_storage_error do
           broken_repository.fetch_ref(broken_repository, source_ref: '1', target_ref: '2')
         end
@@ -1077,7 +1018,7 @@ describe Repository do
         repository.add_branch(project.creator, ref, 'master')
       end
 
-      it 'should be true' do
+      it 'is true' do
         is_expected.to eq(true)
       end
     end
@@ -1087,7 +1028,7 @@ describe Repository do
         repository.add_tag(project.creator, ref, 'master')
       end
 
-      it 'should be false' do
+      it 'is false' do
         is_expected.to eq(false)
       end
     end
@@ -1156,6 +1097,49 @@ describe Repository do
     end
   end
 
+  shared_examples 'asymmetric cached method' do |method|
+    context 'asymmetric caching', :use_clean_rails_memory_store_caching, :request_store do
+      let(:cache) { repository.send(:cache) }
+      let(:request_store_cache) { repository.send(:request_store_cache) }
+
+      context 'when it returns true' do
+        before do
+          expect(repository.raw_repository).to receive(method).once.and_return(true)
+        end
+
+        it 'caches the output in RequestStore' do
+          expect do
+            repository.send(method)
+          end.to change { request_store_cache.read(method) }.from(nil).to(true)
+        end
+
+        it 'caches the output in RepositoryCache' do
+          expect do
+            repository.send(method)
+          end.to change { cache.read(method) }.from(nil).to(true)
+        end
+      end
+
+      context 'when it returns false' do
+        before do
+          expect(repository.raw_repository).to receive(method).once.and_return(false)
+        end
+
+        it 'caches the output in RequestStore' do
+          expect do
+            repository.send(method)
+          end.to change { request_store_cache.read(method) }.from(nil).to(false)
+        end
+
+        it 'does NOT cache the output in RepositoryCache' do
+          expect do
+            repository.send(method)
+          end.not_to change { cache.read(method) }.from(nil)
+        end
+      end
+    end
+  end
+
   describe '#exists?' do
     it 'returns true when a repository exists' do
       expect(repository.exists?).to be(true)
@@ -1168,51 +1152,12 @@ describe Repository do
     end
 
     context 'with broken storage', :broken_storage do
-      it 'should raise a storage error' do
+      it 'raises a storage error' do
         expect_to_raise_storage_error { broken_repository.exists? }
       end
     end
 
-    context 'asymmetric caching', :use_clean_rails_memory_store_caching, :request_store do
-      let(:cache) { repository.send(:cache) }
-      let(:request_store_cache) { repository.send(:request_store_cache) }
-
-      context 'when it returns true' do
-        before do
-          expect(repository.raw_repository).to receive(:exists?).once.and_return(true)
-        end
-
-        it 'caches the output in RequestStore' do
-          expect do
-            repository.exists?
-          end.to change { request_store_cache.read(:exists?) }.from(nil).to(true)
-        end
-
-        it 'caches the output in RepositoryCache' do
-          expect do
-            repository.exists?
-          end.to change { cache.read(:exists?) }.from(nil).to(true)
-        end
-      end
-
-      context 'when it returns false' do
-        before do
-          expect(repository.raw_repository).to receive(:exists?).once.and_return(false)
-        end
-
-        it 'caches the output in RequestStore' do
-          expect do
-            repository.exists?
-          end.to change { request_store_cache.read(:exists?) }.from(nil).to(false)
-        end
-
-        it 'does NOT cache the output in RepositoryCache' do
-          expect do
-            repository.exists?
-          end.not_to change { cache.read(:exists?) }.from(nil)
-        end
-      end
-    end
+    it_behaves_like 'asymmetric cached method', :exists?
   end
 
   describe '#has_visible_content?' do
@@ -1298,6 +1243,27 @@ describe Repository do
     end
   end
 
+  describe '#blobs_at' do
+    let(:empty_repository) { create(:project_empty_repo).repository }
+
+    it 'returns empty array for an empty repository' do
+      # rubocop:disable Style/WordArray
+      expect(empty_repository.blobs_at(['master', 'foobar'])).to eq([])
+      # rubocop:enable Style/WordArray
+    end
+
+    it 'returns blob array for a non-empty repository' do
+      repository.create_file(User.last, 'foobar', 'CONTENT', message: 'message', branch_name: 'master')
+
+      # rubocop:disable Style/WordArray
+      blobs = repository.blobs_at([['master', 'foobar']])
+      # rubocop:enable Style/WordArray
+
+      expect(blobs.first.name).to eq('foobar')
+      expect(blobs.size).to eq(1)
+    end
+  end
+
   describe '#root_ref' do
     it 'returns a branch name' do
       expect(repository.root_ref).to be_an_instance_of(String)
@@ -1311,6 +1277,8 @@ describe Repository do
       repository.root_ref
       repository.root_ref
     end
+
+    it_behaves_like 'asymmetric cached method', :root_ref
   end
 
   describe '#expire_root_ref_cache' do
@@ -1389,34 +1357,23 @@ describe Repository do
 
   describe '#merge' do
     let(:merge_request) { create(:merge_request, source_branch: 'feature', target_branch: 'master', source_project: project) }
-
     let(:message) { 'Test \r\n\r\n message' }
 
-    shared_examples '#merge' do
-      it 'merges the code and returns the commit id' do
-        expect(merge_commit).to be_present
-        expect(repository.blob_at(merge_commit.id, 'files/ruby/feature.rb')).to be_present
-      end
-
-      it 'sets the `in_progress_merge_commit_sha` flag for the given merge request' do
-        merge_commit_id = merge(repository, user, merge_request, message)
-
-        expect(merge_request.in_progress_merge_commit_sha).to eq(merge_commit_id)
-      end
-
-      it 'removes carriage returns from commit message' do
-        merge_commit_id = merge(repository, user, merge_request, message)
-
-        expect(repository.commit(merge_commit_id).message).to eq(message.delete("\r"))
-      end
+    it 'merges the code and returns the commit id' do
+      expect(merge_commit).to be_present
+      expect(repository.blob_at(merge_commit.id, 'files/ruby/feature.rb')).to be_present
     end
 
-    context 'with gitaly' do
-      it_behaves_like '#merge'
+    it 'sets the `in_progress_merge_commit_sha` flag for the given merge request' do
+      merge_commit_id = merge(repository, user, merge_request, message)
+
+      expect(merge_request.in_progress_merge_commit_sha).to eq(merge_commit_id)
     end
 
-    context 'without gitaly', :skip_gitaly_mock do
-      it_behaves_like '#merge'
+    it 'removes carriage returns from commit message' do
+      merge_commit_id = merge(repository, user, merge_request, message)
+
+      expect(repository.commit(merge_commit_id).message).to eq(message.delete("\r"))
     end
 
     def merge(repository, user, merge_request, message)
@@ -1424,6 +1381,29 @@ describe Repository do
     end
   end
 
+  describe '#merge_to_ref' do
+    let(:merge_request) do
+      create(:merge_request, source_branch: 'feature',
+                             target_branch: 'master',
+                             source_project: project)
+    end
+
+    it 'writes merge of source and target to MR merge_ref_path' do
+      merge_commit_id = repository.merge_to_ref(user,
+                                                merge_request.diff_head_sha,
+                                                merge_request,
+                                                merge_request.merge_ref_path,
+                                                'Custom message')
+
+      merge_commit = repository.commit(merge_commit_id)
+
+      expect(merge_commit.message).to eq('Custom message')
+      expect(merge_commit.author_name).to eq(user.name)
+      expect(merge_commit.author_email).to eq(user.commit_email)
+      expect(repository.blob_at(merge_commit.id, 'files/ruby/feature.rb')).to be_present
+    end
+  end
+
   describe '#ff_merge' do
     before do
       repository.add_branch(user, 'ff-target', 'feature~5')
@@ -1453,97 +1433,77 @@ describe Repository do
   end
 
   describe '#revert' do
-    shared_examples 'reverting a commit' do
-      let(:new_image_commit) { repository.commit('33f3729a45c02fc67d00adb1b8bca394b0e761d9') }
-      let(:update_image_commit) { repository.commit('2f63565e7aac07bcdadb654e253078b727143ec4') }
-      let(:message) { 'revert message' }
+    let(:new_image_commit) { repository.commit('33f3729a45c02fc67d00adb1b8bca394b0e761d9') }
+    let(:update_image_commit) { repository.commit('2f63565e7aac07bcdadb654e253078b727143ec4') }
+    let(:message) { 'revert message' }
 
-      context 'when there is a conflict' do
-        it 'raises an error' do
-          expect { repository.revert(user, new_image_commit, 'master', message) }.to raise_error(Gitlab::Git::Repository::CreateTreeError)
-        end
-      end
-
-      context 'when commit was already reverted' do
-        it 'raises an error' do
-          repository.revert(user, update_image_commit, 'master', message)
-
-          expect { repository.revert(user, update_image_commit, 'master', message) }.to raise_error(Gitlab::Git::Repository::CreateTreeError)
-        end
-      end
-
-      context 'when commit can be reverted' do
-        it 'reverts the changes' do
-          expect(repository.revert(user, update_image_commit, 'master', message)).to be_truthy
-        end
-      end
-
-      context 'reverting a merge commit' do
-        it 'reverts the changes' do
-          merge_commit
-          expect(repository.blob_at_branch('master', 'files/ruby/feature.rb')).to be_present
-
-          repository.revert(user, merge_commit, 'master', message)
-          expect(repository.blob_at_branch('master', 'files/ruby/feature.rb')).not_to be_present
-        end
+    context 'when there is a conflict' do
+      it 'raises an error' do
+        expect { repository.revert(user, new_image_commit, 'master', message) }.to raise_error(Gitlab::Git::Repository::CreateTreeError)
       end
     end
 
-    context 'when Gitaly revert feature is enabled' do
-      it_behaves_like 'reverting a commit'
+    context 'when commit was already reverted' do
+      it 'raises an error' do
+        repository.revert(user, update_image_commit, 'master', message)
+
+        expect { repository.revert(user, update_image_commit, 'master', message) }.to raise_error(Gitlab::Git::Repository::CreateTreeError)
+      end
     end
 
-    context 'when Gitaly revert feature is disabled', :disable_gitaly do
-      it_behaves_like 'reverting a commit'
+    context 'when commit can be reverted' do
+      it 'reverts the changes' do
+        expect(repository.revert(user, update_image_commit, 'master', message)).to be_truthy
+      end
+    end
+
+    context 'reverting a merge commit' do
+      it 'reverts the changes' do
+        merge_commit
+        expect(repository.blob_at_branch('master', 'files/ruby/feature.rb')).to be_present
+
+        repository.revert(user, merge_commit, 'master', message)
+        expect(repository.blob_at_branch('master', 'files/ruby/feature.rb')).not_to be_present
+      end
     end
   end
 
   describe '#cherry_pick' do
-    shared_examples 'cherry-picking a commit' do
-      let(:conflict_commit) { repository.commit('c642fe9b8b9f28f9225d7ea953fe14e74748d53b') }
-      let(:pickable_commit) { repository.commit('7d3b0f7cff5f37573aea97cebfd5692ea1689924') }
-      let(:pickable_merge) { repository.commit('e56497bb5f03a90a51293fc6d516788730953899') }
-      let(:message) { 'cherry-pick message' }
+    let(:conflict_commit) { repository.commit('c642fe9b8b9f28f9225d7ea953fe14e74748d53b') }
+    let(:pickable_commit) { repository.commit('7d3b0f7cff5f37573aea97cebfd5692ea1689924') }
+    let(:pickable_merge) { repository.commit('e56497bb5f03a90a51293fc6d516788730953899') }
+    let(:message) { 'cherry-pick message' }
 
-      context 'when there is a conflict' do
-        it 'raises an error' do
-          expect { repository.cherry_pick(user, conflict_commit, 'master', message) }.to raise_error(Gitlab::Git::Repository::CreateTreeError)
-        end
-      end
-
-      context 'when commit was already cherry-picked' do
-        it 'raises an error' do
-          repository.cherry_pick(user, pickable_commit, 'master', message)
-
-          expect { repository.cherry_pick(user, pickable_commit, 'master', message) }.to raise_error(Gitlab::Git::Repository::CreateTreeError)
-        end
-      end
-
-      context 'when commit can be cherry-picked' do
-        it 'cherry-picks the changes' do
-          expect(repository.cherry_pick(user, pickable_commit, 'master', message)).to be_truthy
-        end
-      end
-
-      context 'cherry-picking a merge commit' do
-        it 'cherry-picks the changes' do
-          expect(repository.blob_at_branch('improve/awesome', 'foo/bar/.gitkeep')).to be_nil
-
-          cherry_pick_commit_sha = repository.cherry_pick(user, pickable_merge, 'improve/awesome', message)
-          cherry_pick_commit_message = project.commit(cherry_pick_commit_sha).message
-
-          expect(repository.blob_at_branch('improve/awesome', 'foo/bar/.gitkeep')).not_to be_nil
-          expect(cherry_pick_commit_message).to eq(message)
-        end
+    context 'when there is a conflict' do
+      it 'raises an error' do
+        expect { repository.cherry_pick(user, conflict_commit, 'master', message) }.to raise_error(Gitlab::Git::Repository::CreateTreeError)
       end
     end
 
-    context 'when Gitaly cherry_pick feature is enabled' do
-      it_behaves_like 'cherry-picking a commit'
+    context 'when commit was already cherry-picked' do
+      it 'raises an error' do
+        repository.cherry_pick(user, pickable_commit, 'master', message)
+
+        expect { repository.cherry_pick(user, pickable_commit, 'master', message) }.to raise_error(Gitlab::Git::Repository::CreateTreeError)
+      end
     end
 
-    context 'when Gitaly cherry_pick feature is disabled', :disable_gitaly do
-      it_behaves_like 'cherry-picking a commit'
+    context 'when commit can be cherry-picked' do
+      it 'cherry-picks the changes' do
+        expect(repository.cherry_pick(user, pickable_commit, 'master', message)).to be_truthy
+      end
+    end
+
+    context 'cherry-picking a merge commit' do
+      it 'cherry-picks the changes' do
+        expect(repository.blob_at_branch('improve/awesome', 'foo/bar/.gitkeep')).to be_nil
+
+        cherry_pick_commit_sha = repository.cherry_pick(user, pickable_merge, 'improve/awesome', message)
+        cherry_pick_commit_message = project.commit(cherry_pick_commit_sha).message
+
+        expect(repository.blob_at_branch('improve/awesome', 'foo/bar/.gitkeep')).not_to be_nil
+        expect(cherry_pick_commit_message).to eq(message)
+      end
     end
   end
 
@@ -1641,6 +1601,7 @@ describe Repository do
         :size,
         :commit_count,
         :rendered_readme,
+        :readme_path,
         :contribution_guide,
         :changelog,
         :license_blob,
@@ -2027,6 +1988,42 @@ describe Repository do
     end
   end
 
+  describe '#readme_path', :use_clean_rails_memory_store_caching do
+    context 'with a non-existing repository' do
+      let(:project) { create(:project) }
+
+      it 'returns nil' do
+        expect(repository.readme_path).to be_nil
+      end
+    end
+
+    context 'with an existing repository' do
+      context 'when no README exists' do
+        let(:project) { create(:project, :empty_repo) }
+
+        it 'returns nil' do
+          expect(repository.readme_path).to be_nil
+        end
+      end
+
+      context 'when a README exists' do
+        let(:project) { create(:project, :repository) }
+
+        it 'returns the README' do
+          expect(repository.readme_path).to eq("README.md")
+        end
+
+        it 'caches the response' do
+          expect(repository).to receive(:readme).and_call_original.once
+
+          2.times do
+            expect(repository.readme_path).to eq("README.md")
+          end
+        end
+      end
+    end
+  end
+
   describe '#expire_statistics_caches' do
     it 'expires the caches' do
       expect(repository).to receive(:expire_method_caches)
@@ -2195,9 +2192,10 @@ describe Repository do
   describe '#refresh_method_caches' do
     it 'refreshes the caches of the given types' do
       expect(repository).to receive(:expire_method_caches)
-        .with(%i(rendered_readme license_blob license_key license))
+        .with(%i(rendered_readme readme_path license_blob license_key license))
 
       expect(repository).to receive(:rendered_readme)
+      expect(repository).to receive(:readme_path)
       expect(repository).to receive(:license_blob)
       expect(repository).to receive(:license_key)
       expect(repository).to receive(:license)
@@ -2247,38 +2245,42 @@ describe Repository do
     rugged.references.create("refs/remotes/#{remote_name}/#{branch_name}", target.id)
   end
 
-  describe '#ancestor?' do
+  shared_examples '#ancestor?' do
     let(:commit) { repository.commit }
     let(:ancestor) { commit.parents.first }
 
-    shared_examples '#ancestor?' do
-      it 'it is an ancestor' do
-        expect(repository.ancestor?(ancestor.id, commit.id)).to eq(true)
-      end
-
-      it 'it is not an ancestor' do
-        expect(repository.ancestor?(commit.id, ancestor.id)).to eq(false)
-      end
-
-      it 'returns false on nil-values' do
-        expect(repository.ancestor?(nil, commit.id)).to eq(false)
-        expect(repository.ancestor?(ancestor.id, nil)).to eq(false)
-        expect(repository.ancestor?(nil, nil)).to eq(false)
-      end
-
-      it 'returns false for invalid commit IDs' do
-        expect(repository.ancestor?(commit.id, Gitlab::Git::BLANK_SHA)).to eq(false)
-        expect(repository.ancestor?( Gitlab::Git::BLANK_SHA, commit.id)).to eq(false)
-      end
+    it 'is an ancestor' do
+      expect(repository.ancestor?(ancestor.id, commit.id)).to eq(true)
     end
 
-    context 'with Gitaly enabled' do
-      it_behaves_like('#ancestor?')
+    it 'is not an ancestor' do
+      expect(repository.ancestor?(commit.id, ancestor.id)).to eq(false)
     end
 
-    context 'with Gitaly disabled', :skip_gitaly_mock do
-      it_behaves_like('#ancestor?')
+    it 'returns false on nil-values' do
+      expect(repository.ancestor?(nil, commit.id)).to eq(false)
+      expect(repository.ancestor?(ancestor.id, nil)).to eq(false)
+      expect(repository.ancestor?(nil, nil)).to eq(false)
     end
+
+    it 'returns false for invalid commit IDs' do
+      expect(repository.ancestor?(commit.id, Gitlab::Git::BLANK_SHA)).to eq(false)
+      expect(repository.ancestor?( Gitlab::Git::BLANK_SHA, commit.id)).to eq(false)
+    end
+  end
+
+  describe '#ancestor? with Gitaly enabled' do
+    it_behaves_like "#ancestor?"
+  end
+
+  describe '#ancestor? with Rugged enabled', :enable_rugged do
+    it 'calls out to the Rugged implementation' do
+      allow_any_instance_of(Rugged).to receive(:merge_base).with(repository.commit.id, Gitlab::Git::BLANK_SHA).and_call_original
+
+      repository.ancestor?(repository.commit.id, Gitlab::Git::BLANK_SHA)
+    end
+
+    it_behaves_like '#ancestor?'
   end
 
   describe '#archive_metadata' do
@@ -2334,6 +2336,7 @@ describe Repository do
       expect(subject).to be_a(Gitlab::Git::Repository)
       expect(subject.relative_path).to eq(project.disk_path + '.git')
       expect(subject.gl_repository).to eq("project-#{project.id}")
+      expect(subject.gl_project_path).to eq(project.full_path)
     end
 
     context 'with a wiki repository' do
@@ -2343,6 +2346,7 @@ describe Repository do
         expect(subject).to be_a(Gitlab::Git::Repository)
         expect(subject.relative_path).to eq(project.disk_path + '.wiki.git')
         expect(subject.gl_repository).to eq("wiki-#{project.id}")
+        expect(subject.gl_project_path).to eq(project.full_path)
       end
     end
   end
