@@ -1,3 +1,5 @@
+# frozen_string_literal: true
+
 require "spec_helper"
 
 describe WikiPage do
@@ -18,12 +20,18 @@ describe WikiPage do
     context 'when there are pages' do
       before do
         create_page('dir_1/dir_1_1/page_3', 'content')
-        create_page('dir_1/page_2', 'content')
-        create_page('dir_2/page_5', 'content')
-        create_page('dir_2/page_4', 'content')
         create_page('page_1', 'content')
+        create_page('dir_1/page_2', 'content')
+        create_page('dir_2', 'page with dir name')
+        create_page('dir_2/page_5', 'content')
+        create_page('page_6', 'content')
+        create_page('dir_2/page_4', 'content')
       end
+
       let(:page_1) { wiki.find_page('page_1') }
+      let(:page_6) { wiki.find_page('page_6') }
+      let(:page_dir_2) { wiki.find_page('dir_2') }
+
       let(:dir_1) do
         WikiDirectory.new('dir_1', [wiki.find_page('dir_1/page_2')])
       end
@@ -36,25 +44,38 @@ describe WikiPage do
         WikiDirectory.new('dir_2', pages)
       end
 
-      it 'returns an array with pages and directories' do
-        expected_grouped_entries = [page_1, dir_1, dir_1_1, dir_2]
+      context 'sort by title' do
+        let(:grouped_entries) { described_class.group_by_directory(wiki.pages) }
+        let(:expected_grouped_entries) { [dir_1_1, dir_1, page_dir_2, dir_2, page_1, page_6] }
 
-        grouped_entries = described_class.group_by_directory(wiki.pages)
+        it 'returns an array with pages and directories' do
+          grouped_entries.each_with_index do |page_or_dir, i|
+            expected_page_or_dir = expected_grouped_entries[i]
+            expected_slugs = get_slugs(expected_page_or_dir)
+            slugs = get_slugs(page_or_dir)
 
-        grouped_entries.each_with_index do |page_or_dir, i|
-          expected_page_or_dir = expected_grouped_entries[i]
-          expected_slugs = get_slugs(expected_page_or_dir)
-          slugs = get_slugs(page_or_dir)
-
-          expect(slugs).to match_array(expected_slugs)
+            expect(slugs).to match_array(expected_slugs)
+          end
         end
       end
 
-      it 'returns an array sorted by alphabetical position' do
-        # Directories and pages within directories are sorted alphabetically.
-        # Pages at root come before everything.
-        expected_order = ['page_1', 'dir_1/page_2', 'dir_1/dir_1_1/page_3',
-                          'dir_2/page_4', 'dir_2/page_5']
+      context 'sort by created_at' do
+        let(:grouped_entries) { described_class.group_by_directory(wiki.pages(sort: 'created_at')) }
+        let(:expected_grouped_entries) { [dir_1_1, page_1, dir_1, page_dir_2, dir_2, page_6] }
+
+        it 'returns an array with pages and directories' do
+          grouped_entries.each_with_index do |page_or_dir, i|
+            expected_page_or_dir = expected_grouped_entries[i]
+            expected_slugs = get_slugs(expected_page_or_dir)
+            slugs = get_slugs(page_or_dir)
+
+            expect(slugs).to match_array(expected_slugs)
+          end
+        end
+      end
+
+      it 'returns an array with retained order with directories at the top' do
+        expected_order = ['dir_1/dir_1_1/page_3', 'dir_1/page_2', 'dir_2', 'dir_2/page_4', 'dir_2/page_5', 'page_1', 'page_6']
 
         grouped_entries = described_class.group_by_directory(wiki.pages)
 
@@ -200,180 +221,160 @@ describe WikiPage do
   end
 
   describe '#create' do
-    shared_examples 'create method' do
-      context 'with valid attributes' do
-        it 'raises an error if a page with the same path already exists' do
-          create_page('New Page', 'content')
-          create_page('foo/bar', 'content')
-          expect { create_page('New Page', 'other content') }.to raise_error Gitlab::Git::Wiki::DuplicatePageError
-          expect { create_page('foo/bar', 'other content') }.to raise_error Gitlab::Git::Wiki::DuplicatePageError
+    context 'with valid attributes' do
+      it 'raises an error if a page with the same path already exists' do
+        create_page('New Page', 'content')
+        create_page('foo/bar', 'content')
+        expect { create_page('New Page', 'other content') }.to raise_error Gitlab::Git::Wiki::DuplicatePageError
+        expect { create_page('foo/bar', 'other content') }.to raise_error Gitlab::Git::Wiki::DuplicatePageError
 
-          destroy_page('New Page')
-          destroy_page('bar', 'foo')
-        end
-
-        it 'if the title is preceded by a / it is removed' do
-          create_page('/New Page', 'content')
-
-          expect(wiki.find_page('New Page')).not_to be_nil
-
-          destroy_page('New Page')
-        end
+        destroy_page('New Page')
+        destroy_page('bar', 'foo')
       end
-    end
 
-    context 'when Gitaly is enabled' do
-      it_behaves_like 'create method'
-    end
+      it 'if the title is preceded by a / it is removed' do
+        create_page('/New Page', 'content')
 
-    context 'when Gitaly is disabled', :skip_gitaly_mock do
-      it_behaves_like 'create method'
+        expect(wiki.find_page('New Page')).not_to be_nil
+
+        destroy_page('New Page')
+      end
     end
   end
 
   describe "#update" do
-    shared_examples 'update method' do
-      before do
-        create_page("Update", "content")
+    before do
+      create_page("Update", "content")
+      @page = wiki.find_page("Update")
+    end
+
+    after do
+      destroy_page(@page.title, @page.directory)
+    end
+
+    context "with valid attributes" do
+      it "updates the content of the page" do
+        new_content = "new content"
+
+        @page.update(content: new_content)
         @page = wiki.find_page("Update")
+
+        expect(@page.content).to eq("new content")
       end
 
-      after do
-        destroy_page(@page.title, @page.directory)
+      it "updates the title of the page" do
+        new_title = "Index v.1.2.4"
+
+        @page.update(title: new_title)
+        @page = wiki.find_page(new_title)
+
+        expect(@page.title).to eq(new_title)
       end
 
-      context "with valid attributes" do
-        it "updates the content of the page" do
-          new_content = "new content"
+      it "returns true" do
+        expect(@page.update(content: "more content")).to be_truthy
+      end
+    end
 
-          @page.update(content: new_content)
-          @page = wiki.find_page("Update")
+    context 'with same last commit sha' do
+      it 'returns true' do
+        expect(@page.update(content: 'more content', last_commit_sha: @page.last_commit_sha)).to be_truthy
+      end
+    end
 
-          expect(@page.content).to eq("new content")
-        end
+    context 'with different last commit sha' do
+      it 'raises exception' do
+        expect { @page.update(content: 'more content', last_commit_sha: 'xxx') }.to raise_error(WikiPage::PageChangedError)
+      end
+    end
 
-        it "updates the title of the page" do
-          new_title = "Index v.1.2.4"
+    context 'when renaming a page' do
+      it 'raises an error if the page already exists' do
+        create_page('Existing Page', 'content')
 
-          @page.update(title: new_title)
-          @page = wiki.find_page(new_title)
+        expect { @page.update(title: 'Existing Page', content: 'new_content') }.to raise_error(WikiPage::PageRenameError)
+        expect(@page.title).to eq 'Update'
+        expect(@page.content).to eq 'new_content'
 
-          expect(@page.title).to eq(new_title)
-        end
-
-        it "returns true" do
-          expect(@page.update(content: "more content")).to be_truthy
-        end
+        destroy_page('Existing Page')
       end
 
-      context 'with same last commit sha' do
-        it 'returns true' do
-          expect(@page.update(content: 'more content', last_commit_sha: @page.last_commit_sha)).to be_truthy
-        end
+      it 'updates the content and rename the file' do
+        new_title = 'Renamed Page'
+        new_content = 'updated content'
+
+        expect(@page.update(title: new_title, content: new_content)).to be_truthy
+
+        @page = wiki.find_page(new_title)
+
+        expect(@page).not_to be_nil
+        expect(@page.content).to eq new_content
+      end
+    end
+
+    context 'when moving a page' do
+      it 'raises an error if the page already exists' do
+        create_page('foo/Existing Page', 'content')
+
+        expect { @page.update(title: 'foo/Existing Page', content: 'new_content') }.to raise_error(WikiPage::PageRenameError)
+        expect(@page.title).to eq 'Update'
+        expect(@page.content).to eq 'new_content'
+
+        destroy_page('Existing Page', 'foo')
       end
 
-      context 'with different last commit sha' do
-        it 'raises exception' do
-          expect { @page.update(content: 'more content', last_commit_sha: 'xxx') }.to raise_error(WikiPage::PageChangedError)
-        end
+      it 'updates the content and moves the file' do
+        new_title = 'foo/Other Page'
+        new_content = 'new_content'
+
+        expect(@page.update(title: new_title, content: new_content)).to be_truthy
+
+        page = wiki.find_page(new_title)
+
+        expect(page).not_to be_nil
+        expect(page.content).to eq new_content
       end
 
-      context 'when renaming a page' do
-        it 'raises an error if the page already exists' do
-          create_page('Existing Page', 'content')
-
-          expect { @page.update(title: 'Existing Page', content: 'new_content') }.to raise_error(WikiPage::PageRenameError)
-          expect(@page.title).to eq 'Update'
-          expect(@page.content).to eq 'new_content'
-
-          destroy_page('Existing Page')
-        end
-
-        it 'updates the content and rename the file' do
-          new_title = 'Renamed Page'
-          new_content = 'updated content'
-
-          expect(@page.update(title: new_title, content: new_content)).to be_truthy
-
-          @page = wiki.find_page(new_title)
-
-          expect(@page).not_to be_nil
-          expect(@page.content).to eq new_content
-        end
-      end
-
-      context 'when moving a page' do
-        it 'raises an error if the page already exists' do
+      context 'in subdir' do
+        before do
           create_page('foo/Existing Page', 'content')
-
-          expect { @page.update(title: 'foo/Existing Page', content: 'new_content') }.to raise_error(WikiPage::PageRenameError)
-          expect(@page.title).to eq 'Update'
-          expect(@page.content).to eq 'new_content'
-
-          destroy_page('Existing Page', 'foo')
+          @page = wiki.find_page('foo/Existing Page')
         end
 
-        it 'updates the content and moves the file' do
-          new_title = 'foo/Other Page'
-          new_content = 'new_content'
-
-          expect(@page.update(title: new_title, content: new_content)).to be_truthy
-
-          page = wiki.find_page(new_title)
-
-          expect(page).not_to be_nil
-          expect(page.content).to eq new_content
+        it 'moves the page to the root folder if the title is preceded by /' do
+          expect(@page.slug).to eq 'foo/Existing-Page'
+          expect(@page.update(title: '/Existing Page', content: 'new_content')).to be_truthy
+          expect(@page.slug).to eq 'Existing-Page'
         end
 
-        context 'in subdir' do
-          before do
-            create_page('foo/Existing Page', 'content')
-            @page = wiki.find_page('foo/Existing Page')
-          end
+        it 'does nothing if it has the same title' do
+          original_path = @page.slug
 
-          it 'moves the page to the root folder if the title is preceded by /', :skip_gitaly_mock do
-            expect(@page.slug).to eq 'foo/Existing-Page'
-            expect(@page.update(title: '/Existing Page', content: 'new_content')).to be_truthy
-            expect(@page.slug).to eq 'Existing-Page'
-          end
-
-          it 'does nothing if it has the same title' do
-            original_path = @page.slug
-
-            expect(@page.update(title: 'Existing Page', content: 'new_content')).to be_truthy
-            expect(@page.slug).to eq original_path
-          end
-        end
-
-        context 'in root dir' do
-          it 'does nothing if the title is preceded by /' do
-            original_path = @page.slug
-
-            expect(@page.update(title: '/Update', content: 'new_content')).to be_truthy
-            expect(@page.slug).to eq original_path
-          end
+          expect(@page.update(title: 'Existing Page', content: 'new_content')).to be_truthy
+          expect(@page.slug).to eq original_path
         end
       end
 
-      context "with invalid attributes" do
-        it 'aborts update if title blank' do
-          expect(@page.update(title: '', content: 'new_content')).to be_falsey
-          expect(@page.content).to eq 'new_content'
+      context 'in root dir' do
+        it 'does nothing if the title is preceded by /' do
+          original_path = @page.slug
 
-          page = wiki.find_page('Update')
-          expect(page.content).to eq 'content'
-
-          @page.title = 'Update'
+          expect(@page.update(title: '/Update', content: 'new_content')).to be_truthy
+          expect(@page.slug).to eq original_path
         end
       end
     end
 
-    context 'when Gitaly is enabled' do
-      it_behaves_like 'update method'
-    end
+    context "with invalid attributes" do
+      it 'aborts update if title blank' do
+        expect(@page.update(title: '', content: 'new_content')).to be_falsey
+        expect(@page.content).to eq 'new_content'
 
-    context 'when Gitaly is disabled', :skip_gitaly_mock do
-      it_behaves_like 'update method'
+        page = wiki.find_page('Update')
+        expect(page.content).to eq 'content'
+
+        @page.title = 'Update'
+      end
     end
   end
 
@@ -394,34 +395,24 @@ describe WikiPage do
   end
 
   describe "#versions" do
-    shared_examples 'wiki page versions' do
-      let(:page) { wiki.find_page("Update") }
+    let(:page) { wiki.find_page("Update") }
 
-      before do
-        create_page("Update", "content")
-      end
-
-      after do
-        destroy_page("Update")
-      end
-
-      it "returns an array of all commits for the page" do
-        3.times { |i| page.update(content: "content #{i}") }
-
-        expect(page.versions.count).to eq(4)
-      end
-
-      it 'returns instances of WikiPageVersion' do
-        expect(page.versions).to all( be_a(Gitlab::Git::WikiPageVersion) )
-      end
+    before do
+      create_page("Update", "content")
     end
 
-    context 'when Gitaly is enabled' do
-      it_behaves_like 'wiki page versions'
+    after do
+      destroy_page("Update")
     end
 
-    context 'when Gitaly is disabled', :disable_gitaly do
-      it_behaves_like 'wiki page versions'
+    it "returns an array of all commits for the page" do
+      3.times { |i| page.update(content: "content #{i}") }
+
+      expect(page.versions.count).to eq(4)
+    end
+
+    it 'returns instances of WikiPageVersion' do
+      expect(page.versions).to all( be_a(Gitlab::Git::WikiPageVersion) )
     end
   end
 
@@ -555,23 +546,13 @@ describe WikiPage do
   end
 
   describe '#formatted_content' do
-    shared_examples 'fetching page formatted content' do
-      it 'returns processed content of the page' do
-        subject.create({ title: "RDoc", content: "*bold*", format: "rdoc" })
-        page = wiki.find_page('RDoc')
+    it 'returns processed content of the page' do
+      subject.create({ title: "RDoc", content: "*bold*", format: "rdoc" })
+      page = wiki.find_page('RDoc')
 
-        expect(page.formatted_content).to eq("\n<p><strong>bold</strong></p>\n")
+      expect(page.formatted_content).to eq("\n<p><strong>bold</strong></p>\n")
 
-        destroy_page('RDoc')
-      end
-    end
-
-    context 'when Gitaly wiki_page_formatted_data is enabled' do
-      it_behaves_like 'fetching page formatted content'
-    end
-
-    context 'when Gitaly wiki_page_formatted_data is disabled', :disable_gitaly do
-      it_behaves_like 'fetching page formatted content'
+      destroy_page('RDoc')
     end
   end
 
