@@ -3,7 +3,7 @@
 require 'spec_helper'
 
 describe BlobPresenter, :seed_helper do
-  let(:repository) { Gitlab::Git::Repository.new('default', TEST_REPO_PATH, '') }
+  let(:repository) { Gitlab::Git::Repository.new('default', TEST_REPO_PATH, '', 'group/project') }
 
   let(:git_blob) do
     Gitlab::Git::Blob.find(
