@@ -10,11 +10,13 @@ describe QuickActions::InterpretService do
   let(:milestone) { create(:milestone, project: project, title: '9.10') }
   let(:commit) { create(:commit, project: project) }
   let(:inprogress) { create(:label, project: project, title: 'In Progress') }
+  let(:helmchart) { create(:label, project: project, title: 'Helm Chart Registry') }
   let(:bug) { create(:label, project: project, title: 'Bug') }
   let(:note) { build(:note, commit_id: merge_request.diff_head_sha) }
   let(:service) { described_class.new(project, developer) }
 
   before do
+    stub_licensed_features(multiple_issue_assignees: false)
     project.add_developer(developer)
   end
 
@@ -93,6 +95,26 @@ describe QuickActions::InterpretService do
       end
     end
 
+    shared_examples 'multiword label name starting without ~' do
+      it 'fetches label ids and populates add_label_ids if content contains /label' do
+        helmchart # populate the label
+        _, updates = service.execute(content, issuable)
+
+        expect(updates).to eq(add_label_ids: [helmchart.id])
+      end
+    end
+
+    shared_examples 'label name is included in the middle of another label name' do
+      it 'ignores the sublabel when the content contains the includer label name' do
+        helmchart # populate the label
+        create(:label, project: project, title: 'Chart')
+
+        _, updates = service.execute(content, issuable)
+
+        expect(updates).to eq(add_label_ids: [helmchart.id])
+      end
+    end
+
     shared_examples 'unlabel command' do
       it 'fetches label ids and populates remove_label_ids if content contains /unlabel' do
         issuable.update!(label_ids: [inprogress.id]) # populate the label
@@ -623,6 +645,26 @@ describe QuickActions::InterpretService do
       let(:issuable) { issue }
     end
 
+    it_behaves_like 'multiword label name starting without ~' do
+      let(:content) { %(/label "#{helmchart.title}") }
+      let(:issuable) { issue }
+    end
+
+    it_behaves_like 'multiword label name starting without ~' do
+      let(:content) { %(/label "#{helmchart.title}") }
+      let(:issuable) { merge_request }
+    end
+
+    it_behaves_like 'label name is included in the middle of another label name' do
+      let(:content) { %(/label ~"#{helmchart.title}") }
+      let(:issuable) { issue }
+    end
+
+    it_behaves_like 'label name is included in the middle of another label name' do
+      let(:content) { %(/label ~"#{helmchart.title}") }
+      let(:issuable) { merge_request }
+    end
+
     it_behaves_like 'unlabel command' do
       let(:content) { %(/unlabel ~"#{inprogress.title}") }
       let(:issuable) { issue }
@@ -1222,6 +1264,37 @@ describe QuickActions::InterpretService do
       expect(commands).to be_empty
       expect(text).to eq("#{described_class::SHRUG}\n/close")
     end
+
+    context '/create_merge_request command' do
+      let(:branch_name) { '1-feature' }
+      let(:content) { "/create_merge_request #{branch_name}" }
+      let(:issuable) { issue }
+
+      context 'if issuable is not an Issue' do
+        let(:issuable) { merge_request }
+
+        it_behaves_like 'empty command'
+      end
+
+      context "when logged user cannot create_merge_requests in the project" do
+        let(:project) { create(:project, :archived) }
+
+        it_behaves_like 'empty command'
+      end
+
+      context 'when logged user cannot push code to the project' do
+        let(:project) { create(:project, :private) }
+        let(:service) { described_class.new(project, create(:user)) }
+
+        it_behaves_like 'empty command'
+      end
+
+      it 'populates create_merge_request with branch_name and issue iid' do
+        _, updates = service.execute(content, issuable)
+
+        expect(updates).to eq(create_merge_request: { branch_name: branch_name, issue_iid: issuable.iid })
+      end
+    end
   end
 
   describe '#explain' do
@@ -1473,5 +1546,37 @@ describe QuickActions::InterpretService do
         end
       end
     end
+
+    describe 'create a merge request' do
+      context 'with no branch name' do
+        let(:content) { '/create_merge_request' }
+
+        it 'uses the default branch name' do
+          _, explanations = service.explain(content, issue)
+
+          expect(explanations).to eq(['Creates a branch and a merge request to resolve this issue'])
+        end
+      end
+
+      context 'with a branch name' do
+        let(:content) { '/create_merge_request foo' }
+
+        it 'uses the given branch name' do
+          _, explanations = service.explain(content, issue)
+
+          expect(explanations).to eq(["Creates branch 'foo' and a merge request to resolve this issue"])
+        end
+      end
+    end
+
+    context "#commands_executed_count" do
+      it 'counts commands executed' do
+        content = "/close and \n/assign me and \n/title new title"
+
+        service.execute(content, issue)
+
+        expect(service.commands_executed_count).to eq(3)
+      end
+    end
   end
 end
